
#import "main.h"

@implementation MonkeyView

+(Class)layerClass{
	return [CAEAGLLayer class];
}

-(id)initWithCoder:(NSCoder*)coder{

	if( self=[super initWithCoder:coder] ){
	
		// Enable retina display
#if RETINA_ENABLED
		if( [self respondsToSelector:@selector(contentScaleFactor)] ){
			float scaleFactor=[[UIScreen mainScreen] scale];
			[self setContentScaleFactor:scaleFactor];
		}
#endif
    
		// Get the layer
		CAEAGLLayer *eaglLayer = (CAEAGLLayer *)self.layer;
		      
		eaglLayer.opaque=TRUE;
		eaglLayer.drawableProperties=[NSDictionary dictionaryWithObjectsAndKeys:
        [NSNumber numberWithBool:FALSE],kEAGLDrawablePropertyRetainedBacking,kEAGLColorFormatRGBA8,kEAGLDrawablePropertyColorFormat,nil];
		
		//ES1 renderer...
		context=[[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
		      
		if( !context || ![EAGLContext setCurrentContext:context] ){
			[self release];
			return nil;
		}
		
		// Create default framebuffer object. The backing will be allocated for the current layer in -resizeFromLayer
		glGenFramebuffersOES( 1,&defaultFramebuffer );
		glGenRenderbuffersOES( 1,&colorRenderbuffer );
		glBindFramebufferOES( GL_FRAMEBUFFER_OES,defaultFramebuffer );
		glBindRenderbufferOES( GL_RENDERBUFFER_OES,colorRenderbuffer );
		glFramebufferRenderbufferOES( GL_FRAMEBUFFER_OES,GL_COLOR_ATTACHMENT0_OES,GL_RENDERBUFFER_OES,colorRenderbuffer );
	}
	return self;
}

-(void)drawView:(id)sender{
	MonkeyAppDelegate *delegate=(MonkeyAppDelegate*) [[UIApplication sharedApplication] delegate];
	[delegate drawView:self];
}

-(void)presentRenderbuffer{
	[context presentRenderbuffer:GL_RENDERBUFFER_OES];
}

-(BOOL)resizeFromLayer:(CAEAGLLayer *)layer{

	// Allocate color buffer backing based on the current layer size
	glBindRenderbufferOES( GL_RENDERBUFFER_OES,colorRenderbuffer );
	[context renderbufferStorage:GL_RENDERBUFFER_OES fromDrawable:layer];
	glGetRenderbufferParameterivOES( GL_RENDERBUFFER_OES,GL_RENDERBUFFER_WIDTH_OES,&backingWidth );
	glGetRenderbufferParameterivOES( GL_RENDERBUFFER_OES,GL_RENDERBUFFER_HEIGHT_OES,&backingHeight );
	
	if( glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES) != GL_FRAMEBUFFER_COMPLETE_OES ){
		NSLog( @"Failed to make complete framebuffer object %x",glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES) );
		abort();
	}
    
	return YES;
}

-(void)layoutSubviews{
	[self resizeFromLayer:(CAEAGLLayer*)self.layer];
	
	[self drawView:nil];
}

-(void)dealloc{

	// Tear down GL
	if( defaultFramebuffer ){
		glDeleteFramebuffersOES(1, &defaultFramebuffer);
		defaultFramebuffer = 0;
	}

	if( colorRenderbuffer ){
		glDeleteRenderbuffersOES(1, &colorRenderbuffer);
		colorRenderbuffer = 0;
	}
	
	// Tear down context
	if( [EAGLContext currentContext]==context ){
        [EAGLContext setCurrentContext:nil];
	}

	[context release];
	context = nil;
	
	[super dealloc];
}

@end

@implementation MonkeyWindow

-(void)sendEvent:(UIEvent*)event{
	[super sendEvent:event];
	MonkeyAppDelegate *delegate=(MonkeyAppDelegate*)[[UIApplication sharedApplication] delegate];
	[delegate onEvent:event];
}

/*
-(void)touchesBegan:(NSSet*)touches withEvent:(UIEvent*)event{
	MonkeyAppDelegate *delegate=(MonkeyAppDelegate*)[[UIApplication sharedApplication] delegate];
	[delegate onEvent:event];
}

-(void)touchesMoved:(NSSet*)touches withEvent:(UIEvent*)event{
	MonkeyAppDelegate *delegate=(MonkeyAppDelegate*)[[UIApplication sharedApplication] delegate];
	[delegate onEvent:event];
}

-(void)touchesEnded:(NSSet*)touches withEvent:(UIEvent*)event{
	MonkeyAppDelegate *delegate=(MonkeyAppDelegate*)[[UIApplication sharedApplication] delegate];
	[delegate onEvent:event];
}

-(void)touchesCancelled:(NSSet*)touches withEvent:(UIEvent*)event{
	MonkeyAppDelegate *delegate=(MonkeyAppDelegate*)[[UIApplication sharedApplication] delegate];
	[delegate onEvent:event];
}
*/

@end

@implementation MonkeyViewController

-(BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation{

	CFArrayRef array=(CFArrayRef)CFBundleGetValueForInfoDictionaryKey( CFBundleGetMainBundle(),CFSTR("UISupportedInterfaceOrientations") );
	if( !array ) return NO;
	
	CFRange range={ 0,CFArrayGetCount( array ) };

	switch( interfaceOrientation ){
	case UIInterfaceOrientationPortrait:
		return CFArrayContainsValue( array,range,CFSTR("UIInterfaceOrientationPortrait") );
	case UIInterfaceOrientationPortraitUpsideDown:
		return CFArrayContainsValue( array,range,CFSTR("UIInterfaceOrientationPortraitUpsideDown") );
	case UIInterfaceOrientationLandscapeLeft:
		return CFArrayContainsValue( array,range,CFSTR("UIInterfaceOrientationLandscapeLeft") );
	case UIInterfaceOrientationLandscapeRight:
		return CFArrayContainsValue( array,range,CFSTR("UIInterfaceOrientationLandscapeRight") );
	}
	return NO;
}

@end


//***** MONKEY CODE GOES HERE! *****

//${BEGIN_CODE}

// C++ Monkey runtime.
//
// Placed into the public domain 24/02/2011.
// No warranty implied; use at your own risk.

#include <cmath>
#include <cctype>
#include <cstdio>
#include <cstdlib>
#include <cstring>

#include <vector>
#include <typeinfo>

#if _WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#if _MSC_VER
#define snprintf _snprintf
#endif
#endif

#define DEBUG_GC 1

//***** Util *****

template<class T> T *t_memcpy( T *dst,const T *src,int n ){
	memcpy( dst,src,n*sizeof(T) );
	return dst+n;
}

template<class T> T *t_memset( T *dst,int val,int n ){
	memset( dst,val,n*sizeof(T) );
	return dst+n;
}

template<class T> int t_memcmp( const T *x,const T *y,int n ){
	return memcmp( x,y,n*sizeof(T) );
}

template<class X> int t_strlen( const X *p ){
	const X *t=p++;
	while( *t++ ){}
	return t-p;
}

template<class T> T *t_create( int n,T *p ){
	for( int i=0;i<n;++i ) new(p+i) T();
	return p+n;
}

template<class T> T *t_create( int n,T *p,const T *q ){
	for( int i=0;i<n;++i ) new(p+i) T(q[i]);
	return p+n;
}

template<class T> void t_destroy( int n,T *p ){
	for( int i=0;i<n;++i ) p[i].~T();
}

// ***** gc_object *****

int gc_alloced;

void *gc_malloc( int size ){
#if DEBUG_GC
	int *p=(int*)malloc( sizeof(int)+size );
	if( !p ) return 0;
	gc_alloced+=size;
	*p++=size;
	return p;
#else
	return malloc( size );
#endif
}

void gc_free( void *q ){
#if DEBUG_GC
	int *p=(int*)q-1;
	gc_alloced-=*p;
	free( p );
#else
	free( q );
#endif
}

struct gc_object;

int gc_total;		//total objects allocated
int gc_marked;		//number of objects marked
int gc_markbit;		//toggling mark bit - 1/0

gc_object *gc_objs;

struct gc_object{
	gc_object *succ;
	int flags;

	gc_object(){
		succ=gc_objs;
		gc_objs=this;
		flags=gc_markbit^1;	//starts unmarked
		++gc_total;
	}

	virtual ~gc_object(){
		--gc_total;
	}

	virtual void mark(){
	}

	void *operator new( size_t size ){
		return gc_malloc( size );
	}
};

void gc_mark();

void gc_sweep(){

	gc_object **q=&gc_objs;
	
	while( gc_marked!=gc_total ){

		gc_object *p=*q;

		while( (p->flags & 1)==gc_markbit ){
			q=&p->succ;
			p=*q;
		}

		*q=p->succ;
		p->~gc_object();
		gc_free( p );
	}
	gc_markbit^=1;
	gc_marked=0;
}

void gc_mark( int t ){
}

void gc_mark( float t ){
}

void gc_mark( gc_object *p ){
	if( !p || (p->flags & 1)==gc_markbit ) return;
	p->flags^=1;
	++gc_marked;
	p->mark();
}

// ***** Monkey Types *****

typedef wchar_t Char;
template<class T> class Array;
class String;
class Object;

// ***** Array *****

template<class T> class Array{
public:
	Array():rep( Rep::alloc(0) ){
	}

	Array( const Array &t ):rep( t.rep ){
	}

	Array( int length ):rep( Rep::alloc(length) ){
		t_create( rep->length,rep->data );
	}
	
	Array( const T *p,int length ):rep( Rep::alloc(length) ){
		t_create( rep->length,rep->data,p );
	}
	
	~Array(){
	}

	Array &operator=( const Array &t ){
		rep=t.rep;
		return *this;
	}
	
	int Length()const{ 
		return rep->length; 
	}
	
	T &At( int index ){
		if( index<0 || index>=rep->length ) throw "Array index out of range";
		return rep->data[index]; 
	}
	
	const T &At( int index )const{
		if( index<0 || index>=rep->length ) throw "Array index out of range";
		return rep->data[index]; 
	}
	
	T &operator[]( int index ){ 
		return rep->data[index]; 
	}

	const T &operator[]( int index )const{
		return rep->data[index]; 
	}
	
	Array Slice( int from,int term )const{
		int len=rep->length;
		if( from<0 ){ 
			from+=len;
			if( from<0 ) from=0;
		}else if( from>len ){
			from=len;
		}
		if( term<0 ){
			term+=len;
		}else if( term>len ){
			term=len;
		}
		if( term<=from ) return Array();
		return Array( rep->data+from,term-from );
	}

	Array Slice( int from )const{
		return Slice( from,rep->length );
	}
	
	Array Resize( int newlen )const{
		if( newlen<=0 ) return Array();
		int n=rep->length;
		if( newlen<n ) n=newlen;
		Rep *p=Rep::alloc( newlen );
		T *q=p->data;
		q=t_create( n,q,rep->data );
		q=t_create( (newlen-n),q );
		return Array( p );
	}

private:
	struct Rep : public gc_object{
		int length;
		T data[0];
		
		Rep( int length ):length(length){
		}
		
		~Rep(){
			for( int i=0;i<length;++i ) data[i].~T();
		}

		void mark(){
			for( int i=0;i<length;++i ) gc_mark( data[i] );
		}

		static Rep *alloc( int length ){
			void *p=gc_malloc( sizeof(Rep)+length*sizeof(T) );
			return ::new(p) Rep( length );
		}
	};
	Rep *rep;

	template<class C> friend void gc_mark( Array<C> &t );
	
	Array( Rep *rep ):rep(rep){
	}
};

template<class T> void gc_mark( Array<T> &t ){
	gc_mark( t.rep );
}

// ***** String *****

class String{
public:
	String():rep( Rep::alloc(0) ){
	}
	
	String( const String &t ):rep( t.rep ){
		rep->retain();
	}

	String( int n ){
		char buf[16];
		snprintf( buf,16,"%i",n );
		rep=Rep::alloc( t_strlen(buf) );
		for( int i=0;i<rep->length;++i ) rep->data[i]=buf[i];
	}

	String( float n ){
		char buf[16];
		snprintf( buf,16,"%f",n );
		rep=Rep::alloc( t_strlen(buf) );
		for( int i=0;i<rep->length;++i ) rep->data[i]=buf[i];
	}

	String( Char ch,int length ):rep( Rep::alloc(length) ){
		for( int i=0;i<length;++i ) rep->data[i]=ch;
	}

	String( const Char *p ):rep( Rep::alloc(t_strlen(p)) ){
		t_memcpy( rep->data,p,rep->length );
	}

	String( const Char *p,int length ):rep( Rep::alloc(length) ){
		t_memcpy( rep->data,p,rep->length );
	}
	
#if __OBJC__	
	String( NSString *nsstr ):rep( Rep::alloc([nsstr length]) ){
		unichar *buf=(unichar*)malloc( rep->length * sizeof(unichar) );
		[nsstr getCharacters:buf range:NSMakeRange(0,rep->length)];
		for( int i=0;i<rep->length;++i ) rep->data[i]=buf[i];
		free( buf );
	}
#endif
	
	~String(){
		rep->release();
	}
	
	template<class C> String( const C *p ):rep( Rep::alloc(t_strlen(p)) ){
		for( int i=0;i<rep->length;++i ) rep->data[i]=p[i];
	}
	
	template<class C> String( const C *p,int length ):rep( Rep::alloc(length) ){
		for( int i=0;i<rep->length;++i ) rep->data[i]=p[i];
	}
	
	int Length()const{
		return rep->length;
	}
	
	const Char *Data()const{
		return rep->data;
	}
	
	Char operator[]( int index )const{
		return rep->data[index];
	}
	
	String &operator=( const String &t ){
		t.rep->retain();
		rep->release();
		rep=t.rep;
		return *this;
	}
	
	String &operator+=( const String &t ){
		return operator=( *this+t );
	}
	
	int Compare( const String &t )const{
		int n=rep->length<t.rep->length ? rep->length : t.rep->length;
		for( int i=0;i<n;++i ){
			if( int q=(int)(rep->data[i])-(int)(t.rep->data[i]) ) return q;
		}
		return rep->length-t.rep->length;
	}
	
	bool operator==( const String &t )const{
		return rep->length==t.rep->length && t_memcmp( rep->data,t.rep->data,rep->length )==0;
	}
	
	bool operator!=( const String &t )const{
		return rep->length!=t.rep->length || t_memcmp( rep->data,t.rep->data,rep->length )!=0;
	}
	
	bool operator<( const String &t )const{
		return Compare( t )<0;
	}
	
	bool operator<=( const String &t )const{
		return Compare( t )<=0;
	}
	
	bool operator>( const String &t )const{
		return Compare( t )>0;
	}
	
	bool operator>=( const String &t )const{
		return Compare( t )>=0;
	}
	
	String operator+( const String &t )const{
		if( !rep->length ) return t;
		if( !t.rep->length ) return *this;
		Rep *p=Rep::alloc( rep->length+t.rep->length );
		Char *q=p->data;
		q=t_memcpy( q,rep->data,rep->length );
		q=t_memcpy( q,t.rep->data,t.rep->length );
		return String( p );
	}
	
	int Find( String find,int start=0 )const{
		if( start<0 ) start=0;
		while( start+find.rep->length<=rep->length ){
			if( !t_memcmp( rep->data+start,find.rep->data,find.rep->length ) ) return start;
			++start;
		}
		return -1;
	}
	
	int FindLast( String find )const{
		int start=rep->length-find.rep->length;
		while( start>=0 ){
			if( !t_memcmp( rep->data+start,find.rep->data,find.rep->length ) ) return start;
			--start;
		}
		return -1;
	}
	
	int FindLast( String find,int start )const{
		if( start>rep->length-find.rep->length ) start=rep->length-find.rep->length;
		while( start>=0 ){
			if( !t_memcmp( rep->data+start,find.rep->data,find.rep->length ) ) return start;
			--start;
		}
		return -1;
	}
	
	String Trim()const{
		int i=0,i2=rep->length;
		while( i<i2 && rep->data[i]<=32 ) ++i;
		while( i2>i && rep->data[i2-1]<=32 ) --i2;
		if( i==0 && i2==rep->length ) return *this;
		return String( rep->data+i,i2-i );
	}

	Array<String> Split( String sep )const{
		if( !sep.rep->length ){
			return Array<String>();
		}
		int i=0,i2,n=1;
		while( (i2=Find( sep,i ))!=-1 ){
			++n;
			i=i2+sep.rep->length;
		}
		Array<String> bits( n );
		if( n==1 ){
			bits[0]=*this;
			return bits;
		}
		i=0;n=0;
		while( (i2=Find( sep,i ))!=-1 ){
			bits[n++]=Slice( i,i2 );
			i=i2+sep.rep->length;
		}
		bits[n]=Slice( i );
		return bits;
	}

	String Join( Array<String> bits )const{
		if( bits.Length()==0 ) return String();
		if( bits.Length()==1 ) return bits[0];
		int newlen=rep->length * (bits.Length()-1);
		for( int i=0;i<bits.Length();++i ){
			newlen+=bits[i].rep->length;
		}
		Rep *p=Rep::alloc( newlen );
		Char *q=p->data;
		q=t_memcpy( q,bits[0].rep->data,bits[0].rep->length );
		for( int i=1;i<bits.Length();++i ){
			q=t_memcpy( q,rep->data,rep->length );
			q=t_memcpy( q,bits[i].rep->data,bits[i].rep->length );
		}
		return String( p );
	}

	String Replace( String find,String repl )const{
		int i=0,i2,newlen=0;
		while( (i2=Find( find,i ))!=-1 ){
			newlen+=(i2-i)+repl.rep->length;
			i=i2+find.rep->length;
		}
		if( !i ) return *this;
		newlen+=rep->length-i;
		Rep *p=Rep::alloc( newlen );
		Char *q=p->data;
		i=0;
		while( (i2=Find( find,i ))!=-1 ){
			q=t_memcpy( q,rep->data+i,i2-i );
			q=t_memcpy( q,repl.rep->data,repl.rep->length );
			i=i2+find.rep->length;
		}
		q=t_memcpy( q,rep->data+i,rep->length-i );
		return String( p );
	}

	String ToLower()const{
		for( int i=0;i<rep->length;++i ){
			Char t=tolower( rep->data[i] );
			if( t==rep->data[i] ) continue;
			Rep *p=Rep::alloc( rep->length );
			Char *q=p->data;
			t_memcpy( q,rep->data,i );
			for( q[i++]=t;i<rep->length;++i ){
				q[i]=tolower( rep->data[i] );
			}
			return String( p );
		}
		return *this;
	}

	String ToUpper()const{
		for( int i=0;i<rep->length;++i ){
			Char t=toupper( rep->data[i] );
			if( t==rep->data[i] ) continue;
			Rep *p=Rep::alloc( rep->length );
			Char *q=p->data;
			t_memcpy( q,rep->data,i );
			for( q[i++]=t;i<rep->length;++i ){
				q[i]=toupper( rep->data[i] );
			}
			return String( p );
		}
		return *this;
	}
	
	bool Contains( String sub )const{
		return Find( sub )!=-1;
	}

	bool StartsWith( String sub )const{
		return sub.rep->length<=rep->length && !t_memcmp( rep->data,sub.rep->data,sub.rep->length );
	}

	bool EndsWith( String sub )const{
		return sub.rep->length<=rep->length && !t_memcmp( rep->data+rep->length-sub.rep->length,sub.rep->data,sub.rep->length );
	}

	String Slice( int from,int term )const{
		int len=rep->length;
		if( from<0 ){
			from+=len;
			if( from<0 ) from=0;
		}else if( from>len ){
			from=len;
		}
		if( term<0 ){
			term+=len;
		}else if( term>len ){
			term=len;
		}
		if( term<from ) return String();
		if( from==0 && term==len ) return *this;
		return String( rep->data+from,term-from );
	}

	String Slice( int from )const{
		return Slice( from,rep->length );
	}
	
	int ToInt()const{
		return atoi( ToCString<char>() );
	}
	
	float ToFloat()const{
		return atof( ToCString<char>() );
	}
	
	template<class C> C *ToCString()const{
		C *p=(C*)gc_malloc( (rep->length+1) * sizeof(C) );
		for( int i=0;i<rep->length;++i ) p[i]=rep->data[i];
		p[rep->length]=0;
		return p;
	}

#if __OBJC__	
	NSString *ToNSString()const{
		return [NSString stringWithCharacters:ToCString<unichar>() length:rep->length];
	}
#endif
	
private:
	struct Rep{
		int refs;
		int length;
		Char data[0];
		
		Rep( int length ):refs(1),length(length){
		}
		
		void retain(){
			++refs;
		}
		
		void release(){
			if( --refs || !length ) return;
			gc_free( this );
		}

		static Rep *alloc( int length ){
			if( !length ){
				static Rep null(0);
				return &null;
			}
			void *p=gc_malloc( sizeof(Rep)+length*sizeof(Char) );
			return new(p) Rep( length );
		}
	};
	Rep *rep;
	
	String( Rep *rep ):rep(rep){
	}
};

void gc_mark( String &t ){
}

String T( const char *p ){
	return String( p );
}

// ***** Object *****

class Object : public gc_object{
public:
};

// ***** print *****

//**** main ****

int argc;
const char **argv;
const char *errInfo="";
std::vector<const char*> errStack;

void pushErr(){
	errStack.push_back( errInfo );
}

void popErr(){
	errInfo=errStack.back();
	errStack.pop_back();
}

String StackTrace(){
	String str;
	pushErr();
	for( int i=errStack.size()-1;i>=0;--i ){
		str+=String( errStack[i] )+"\n";
	}
	popErr();
	return str;
}

#if 0	//def _MSC_VER

void Print( String t ){
	OutputDebugString( (t+"\n").ToCString<TCHAR>() );
}

#else

void Print( String t ){
	puts( t.ToCString<char>() );
	fflush( stdout );
}

#endif

void Error( String t ){
	throw t.ToCString<char>();
}

#if _WIN32

void Alert( String t ){
	MessageBoxA( GetDesktopWindow(),t.ToCString<char>(),"Monkey Error!",MB_OK );
	exit(-1);
}

#elif __OBJC__	//a little misleading...here, we really want 'iOS'...but what is it?

void Alert( String t ){
	UIAlertView *aview=[[UIAlertView alloc] 
	initWithTitle:@"Monkey Error" 
	message:t.ToNSString() 
	delegate:nil
	cancelButtonTitle:nil
	otherButtonTitles:nil];
	[aview autorelease];
	[aview show];
}

#else

void Alert( String t ){
	Print( t );
}

#endif

void Die( const char *p ){
	if( !p || !*p ) exit( 0 );
	String t=String(errInfo)+" : Error : "+p;
	puts( t.ToCString<char>() );fflush( stdout );
	Alert( String(p)+"\n"+StackTrace() );
#if !__OBJC__
	exit(-1);
#endif
}

void sighandler( int sig  ){
	switch( sig ){
	case SIGILL:throw "Illegal instruction";
	case SIGFPE:throw "Floating point exception";
#if __APPLE__	
	case SIGBUS:	//on mac...
#endif
	case SIGSEGV:throw "Null object access";
	}
	throw "SIG???";
}

int bb_Init();
int bb_Main();

//entry point call by target main()...
//
int bb_std_main( int argc,const char **argv ){
	
	::argc=argc;
	::argv=argv;

	signal( SIGILL,sighandler );
	signal( SIGFPE,sighandler );
#if __APPLE__
	signal( SIGBUS,sighandler );
#endif
	signal( SIGSEGV,sighandler );

	try{
		bb_Init();
		bb_Main();
	}catch( const char *p ){
		Die( p );
	}
	
	return 0;
}

// iOS mojo runtime.
//
// Copyright 2011 Mark Sibly, all rights reserved.
// No warranty implied; use at your own risk.

class gxtkApp;
class gxtkGraphics;
class gxtkSurface;
class gxtkInput;
class gxtkAudio;
class gxtkSample;

#include <mach/mach_time.h>

#define KEY_LMB 1
#define KEY_TOUCH0 0x180

#define MAX_CHANNELS 32

gxtkApp *app;

String tostring( NSString *nsstr ){
	return String( [nsstr UTF8String] );
}

NSString *tonsstr( String str ){
	return [NSString stringWithUTF8String:str.ToCString<char>() ];
}

int Pow2Size( int n ){
	int i=1;
	while( i<n ) i*=2;
	return i;
}

BOOL CheckForExtension( NSString *name ){
	static NSArray *extensions;
	if( !extensions ){
		NSString *extensionsString=[NSString stringWithCString:(const char*)glGetString(GL_EXTENSIONS) encoding:NSASCIIStringEncoding];
		extensions=[extensionsString componentsSeparatedByString:@" "];
		[extensions retain];	//?Really needed?
	}
	return [extensions containsObject:name];
}

class gxtkObject : public Object{
public:
};

class gxtkApp : public gxtkObject{
public:
	MonkeyAppDelegate *appDelegate;

	gxtkGraphics *graphics;
	gxtkInput *input;
	gxtkAudio *audio;
	
	uint64_t startTime;
	mach_timebase_info_data_t timeInfo;
	
	int created;
	int updateRate;
	NSTimer *updateTimer;
	
	gxtkApp();
	
	virtual int InvokeOnCreate();
	virtual int InvokeOnUpdate();
	virtual int InvokeOnSuspend();
	virtual int InvokeOnResume();
	virtual int InvokeOnRender();

	//***** GXTK API *****

	virtual gxtkGraphics *GraphicsDevice();
	virtual gxtkInput *InputDevice();
	virtual gxtkAudio *AudioDevice();
	virtual String AppTitle();
	virtual String LoadState();
	virtual int SaveState( String state );
	virtual String LoadString( String path );
	virtual int SetUpdateRate( int hertz );
	virtual int MilliSecs();
	virtual int Loading();
	virtual int OnCreate();
	virtual int OnUpdate();
	virtual int OnSuspend();
	virtual int OnResume();
	virtual int OnRender();
	virtual int OnLoading();
};

//***** START OF COMMON OPENGL CODE *****

#define MAX_VERTS	1024

struct Vertex{
	float x,y;
	float u,v;
	unsigned c;
};

class gxtkGraphics : public gxtkObject{
public:

	int width;
	int height;

	Vertex verts[MAX_VERTS];
	int color,acolor;
	float r,g,b,alpha;
	float matrix[16];
	bool tformed;

	int vertCount;
	int primType;
	GLuint texture;
	
	gxtkGraphics();
	
	bool Validate();		
	void BeginRender();
	void EndRender();
	void Flush();
	void AddVertex( float x,float y );
	void AddVertex( float x,float y,float u,float v );
	
	//***** GXTK API *****
	virtual int Width();
	virtual int Height();

	virtual gxtkSurface *LoadSurface( String path );
	virtual int DestroySurface( gxtkSurface *surface );
	
	virtual int Cls( float r,float g,float b );
	virtual int SetAlpha( float alpha );
	virtual int SetColor( float r,float g,float b );
	virtual int SetBlend( int blend );
	virtual int SetScissor( int x,int y,int w,int h );
	virtual int SetMatrix( float ix,float iy,float jx,float jy,float tx,float ty );
	
	virtual int DrawRect( float x,float y,float w,float h );
	virtual int DrawLine( float x1,float y1,float x2,float y2 );
	virtual int DrawOval( float x1,float y1,float x2,float y2 );
	virtual int DrawSurface( gxtkSurface *surface,float x,float y );
	virtual int DrawSurface2( gxtkSurface *surface,float x,float y,int srcx,int srcy,int srcw,int srch );
};

//***** gxtkSurface *****

class gxtkSurface : public gxtkObject{
public:
	GLuint texture;
	int width;
	int height;
	float uscale;
	float vscale;
	
	gxtkSurface( GLuint texture,int width,int height,float uscale,float vscale );
	
	//***** GXTK API *****
	virtual int Width();
	virtual int Height();
	virtual int Loaded();

};

//***** gxtkGraphics *****

gxtkGraphics::gxtkGraphics(){
	width=height=0;
	memset( matrix,0,sizeof(matrix) );
	matrix[0]=matrix[5]=matrix[10]=matrix[15]=1;
}

void gxtkGraphics::BeginRender(){
	glViewport( 0,0,width,height );

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glOrthof( 0,width,height,0,-1,1 );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	
	glEnableClientState( GL_VERTEX_ARRAY );
	glVertexPointer( 2,GL_FLOAT,sizeof(Vertex),&verts[0].x );	
	
	glEnableClientState( GL_TEXTURE_COORD_ARRAY );
	glTexCoordPointer( 2,GL_FLOAT,sizeof(Vertex),&verts[0].u );
	
	glEnableClientState( GL_COLOR_ARRAY );
	glColorPointer( 4,GL_UNSIGNED_BYTE,sizeof(Vertex),&verts[0].c );
	
	glDisable( GL_TEXTURE_2D );

	glEnable( GL_BLEND );
	glBlendFunc( GL_ONE,GL_ONE_MINUS_SRC_ALPHA );
	
	vertCount=0;
}

void gxtkGraphics::Flush(){
	if( !vertCount ) return;

	if( texture ){
		glEnable( GL_TEXTURE_2D );
		glBindTexture( GL_TEXTURE_2D,texture );
	}
		
	switch( primType ){
	case 2:
		glDrawArrays( GL_LINES,0,vertCount );
		break;
	case 3:
		glDrawArrays( GL_TRIANGLES,0,vertCount );
		break;
	case 5:
		glDrawArrays( GL_TRIANGLE_FAN,0,vertCount );
		break;
	}

	if( texture ){
		glDisable( GL_TEXTURE_2D );
	}
	
	vertCount=0;
}

void gxtkGraphics::AddVertex( float x,float y ){
	verts[vertCount].x=x;
	verts[vertCount].y=y;
	verts[vertCount++].c=color;
}

void gxtkGraphics::AddVertex( float x,float y,float u,float v ){
	verts[vertCount].x=x;
	verts[vertCount].y=y;
	verts[vertCount].u=u;
	verts[vertCount].v=v;
	verts[vertCount++].c=acolor;
}

//***** GXTK API *****

int gxtkGraphics::Width(){
	return width;
}

int gxtkGraphics::Height(){
	return height;
}

int gxtkGraphics::Cls( float r,float g,float b ){
	vertCount=0;

	glClearColor( r/255.0f,g/255.0f,b/255.0f,1 );
	glClear( GL_COLOR_BUFFER_BIT );

	return 0;
}

int gxtkGraphics::SetAlpha( float alpha ){

	//Flushless!

	this->alpha=alpha;
	
	int a=int(alpha*255);
	
	color=(a<<24) | (int(b*alpha)<<16) | (int(g*alpha)<<8) | int(r*alpha);
	
	acolor=(a<<24) | (a<<16) | (a<<8) | a;
	
	return 0;
}

int gxtkGraphics::SetColor( float r,float g,float b ){

	//Flushless!
	
	this->r=r;
	this->g=g;
	this->b=b;

	int a=int(alpha*255);
	
	color=(a<<24) | (int(b*alpha)<<16) | (int(g*alpha)<<8) | int(r*alpha);
	
	acolor=(a<<24) | (a<<16) | (a<<8) | a;
	
	return 0;
}

int gxtkGraphics::SetBlend( int blend ){

	Flush();
	
	switch( blend ){
	case 1:
		glBlendFunc( GL_ONE,GL_ONE );
		break;
	default:
		glBlendFunc( GL_ONE,GL_ONE_MINUS_SRC_ALPHA );
	}

	return 0;
}

int gxtkGraphics::SetScissor( int x,int y,int w,int h ){

	Flush();
	
	if( x!=0 || y!=0 || w!=Width() || h!=Height() ){
		glEnable( GL_SCISSOR_TEST );
		y=Height()-y-h;
		glScissor( x,y,w,h );
	}else{
		glDisable( GL_SCISSOR_TEST );
	}
	return 0;
}

int gxtkGraphics::SetMatrix( float ix,float iy,float jx,float jy,float tx,float ty ){

	Flush();
	
	tformed=(ix!=1 || iy!=0 || jx!=0 || jy!=1 || tx!=0 || ty!=0);
	matrix[0]=ix;
	matrix[1]=iy;
	matrix[4]=jx;
	matrix[5]=jy;
	matrix[12]=tx;
	matrix[13]=ty;
	
	if( tformed ){
		glLoadMatrixf( matrix );
	}else{
		glLoadIdentity();
	}
	return 0;
}

int gxtkGraphics::DrawLine( float x1,float y1,float x2,float y2 ){

	if( vertCount==0 || primType!=2 || vertCount+2>MAX_VERTS || texture ){
		Flush();
		primType=2;
		texture=0;
	}
	
	AddVertex( x1,y1 );
	AddVertex( x2,y2 );

	return 0;
}
	
int gxtkGraphics::DrawRect( float x,float y,float w,float h ){
	
	if( vertCount==0 || primType!=3 || vertCount+6>MAX_VERTS || texture ){
		Flush();
		primType=3;
		texture=0;
	}
	
	AddVertex( x,y );
	AddVertex( x+w,y );
	AddVertex( x+w,y+h );
	
	AddVertex( x,y );
	AddVertex( x+w,y+h );
	AddVertex( x,y+h );

	return 0;
}

int gxtkGraphics::DrawOval( float x,float y,float w,float h ){
	
	Flush();
	
	primType=5;
	texture=0;
	
	float xr=w/2.0f;
	float yr=h/2.0f;

	int segs;	
	if( tformed ){
		float xx=xr*matrix[0],xy=xr*matrix[1],xd=sqrt(xx*xx+xy*xy);
		float yx=yr*matrix[4],yy=yr*matrix[5],yd=sqrt(yx*yx+yy*yy);
		segs=int( xd+yd );
	}else{
		segs=int( abs(xr)+abs(yr) );
	}

	if( segs>MAX_VERTS ){
		segs=MAX_VERTS;
	}else if( segs<12 ){
		segs=12;
	}else{
		segs&=~3;
	}
	
	x+=xr;
	y+=yr;

	for( int i=0;i<segs;++i ){
		float th=i * 6.28318531f / segs;
		AddVertex( x+cos(th)*xr,y+sin(th)*yr );
	}
	
	return 0;
}

int gxtkGraphics::DestroySurface( gxtkSurface *surface ){
	glDeleteTextures( 1,&surface->texture );
	return 0;
}

int gxtkGraphics::DrawSurface( gxtkSurface *surface,float x,float y ){

	if( vertCount==0 || primType!=3 || vertCount+6>MAX_VERTS || texture!=surface->texture ){
		Flush();
		primType=3;
		texture=surface->texture;
	}
		
	float w=surface->width;
	float h=surface->height;
	float u0=0,u1=w * surface->uscale;
	float v0=0,v1=h * surface->vscale;

	AddVertex( x,y,u0,v0 );
	AddVertex( x+w,y,u1,v0 );
	AddVertex( x+w,y+h,u1,v1 );
	
	AddVertex( x,y,u0,v0 );
	AddVertex( x+w,y+h,u1,v1 );
	AddVertex( x,y+h,u0,v1 );

	return 0;
}

int gxtkGraphics::DrawSurface2( gxtkSurface *surface,float x,float y,int srcx,int srcy,int srcw,int srch ){

	if( vertCount==0 || primType!=3 || vertCount+6>MAX_VERTS || texture!=surface->texture ){
		Flush();
		primType=3;
		texture=surface->texture;
	}
		
	float w=srcw;
	float h=srch;
	float u0=srcx * surface->uscale,u1=(srcx+srcw) * surface->uscale;
	float v0=srcy * surface->vscale,v1=(srcy+srch) * surface->vscale;

	AddVertex( x,y,u0,v0 );
	AddVertex( x+w,y,u1,v0 );
	AddVertex( x+w,y+h,u1,v1 );
	
	AddVertex( x,y,u0,v0 );
	AddVertex( x+w,y+h,u1,v1 );
	AddVertex( x,y+h,u0,v1 );

	return 0;
}

//***** gxtkSurface *****

gxtkSurface::gxtkSurface( GLuint texture,int width,int height,float uscale,float vscale ):
	texture(texture),width(width),height(height),uscale(uscale),vscale(vscale){
}

int gxtkSurface::Width(){
	return width;
}

int gxtkSurface::Height(){
	return height;
}

int gxtkSurface::Loaded(){
	return 1;
}

//***** END OF COMMON OPENGL CODE *****

bool gxtkGraphics::Validate(){
	width=app->appDelegate->view->backingWidth;
	height=app->appDelegate->view->backingHeight;
	return width>0 && height>0;
}

void gxtkGraphics::EndRender(){
	Flush();
	[app->appDelegate->view presentRenderbuffer];
}

gxtkSurface *gxtkGraphics::LoadSurface( String path ){

	//Lawks a lordy! Had a real hassle with attempting to use stringWithCharacters and towstr...gave up for now.
	//
	path=String( L"data/" )+path;
	NSString *nspath=tonsstr( path );
	
	UIImage *uiimage=[ UIImage imageNamed:nspath ];
	if( !uiimage ) return 0;
	
	CGImageRef cgimage=uiimage.CGImage;
	
	int width=CGImageGetWidth( cgimage );
	int height=CGImageGetHeight( cgimage );
	
	int texwidth,texheight;
	
	if( CheckForExtension( @"GL_APPLE_texture_2D_limited_npot" ) ){
		texwidth=width;
		texheight=height;
	}else{
		texwidth=Pow2Size( width );
		texheight=Pow2Size( height );
	}
	
	float uscale=1.0f/texwidth;
	float vscale=1.0f/texheight;

	void *data=calloc( texwidth*texheight,4 );
	
	CGContextRef context=CGBitmapContextCreate( data,width,height,8,texwidth*4,CGImageGetColorSpace(cgimage),kCGImageAlphaPremultipliedLast );
	CGContextDrawImage( context,CGRectMake(0,0,width,height),cgimage );
	CGContextRelease( context );
	
	GLuint texture;
	glGenTextures( 1,&texture );
	glBindTexture( GL_TEXTURE_2D,texture );

	glTexParameteri( GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR );
	
	glTexParameteri( GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE );
	glTexParameteri( GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE );
	
	glTexImage2D( GL_TEXTURE_2D,0,GL_RGBA,texwidth,texheight,0,GL_RGBA,GL_UNSIGNED_BYTE,data );
	
	free( data );
	
	return new gxtkSurface( texture,width,height,uscale,vscale );
}

//***** END OF GRAPHICS *****


class gxtkInput : public gxtkObject{
public:
	int keyStates[512];
	UITouch *touches[32];
	float touchX[32];
	float touchY[32];
	float accelX,accelY,accelZ;
	
	gxtkInput();
	
	void OnEvent( UIEvent *event );
	void OnAcceleration( UIAcceleration *accel );
	
	void BeginUpdate();
	void EndUpdate();
	
	//***** GXTK API *****
	virtual int KeyDown( int key );
	virtual int KeyHit( int key );
	virtual int GetChar();
	
	virtual float MouseX();
	virtual float MouseY();

	virtual float JoyX( int index );
	virtual float JoyY( int index );
	virtual float JoyZ( int index );

	virtual float TouchX( int index );
	virtual float TouchY( int index );
	
	virtual float AccelX();
	virtual float AccelY();
	virtual float AccelZ();
};

class gxtkAudio : public gxtkObject{
public:

	ALuint al_sources[MAX_CHANNELS];
	int nextSource;

	gxtkAudio();

	virtual int AL_Source( int channel );
	
	//***** GXTK API *****
	virtual gxtkSample *LoadSample( String path );
	virtual int DestroySample( gxtkSample *sample );
	virtual int PlaySample( gxtkSample *sample,int channel,int flags );
	virtual int StopChannel( int channel );
	virtual int ChannelState( int channel );
	virtual int SetVolume( int channel,float volume );
	virtual int SetPan( int channel,float pan );
	virtual int SetRate( int channel,float rate );
};

class gxtkSample : public gxtkObject{
public:
	ALuint al_buffer;
	
	gxtkSample( String path );
	
	//***** GXTK API *****
};

//***** gxtkApp *****

gxtkApp::gxtkApp(){
	app=this;
	
	appDelegate=(MonkeyAppDelegate*)[[UIApplication sharedApplication] delegate];
	
	graphics=new gxtkGraphics;
	input=new gxtkInput;
	audio=new gxtkAudio;
	
	mach_timebase_info( &timeInfo );
	startTime=mach_absolute_time();
	
	created=0;
	updateRate=0;
	updateTimer=0;
}

int gxtkApp::InvokeOnCreate(){
	Die( "Interal Error" );
	return 0;
}

int gxtkApp::InvokeOnUpdate(){
	try{
		if( graphics->Validate() ){
			input->BeginUpdate();
			OnUpdate();
			input->EndUpdate();
		}
	}catch( const char *p ){
		Die( p );
	}
	return 0;
}

int gxtkApp::InvokeOnSuspend(){
	try{
		if( updateRate ){
			int upr=updateRate;
			SetUpdateRate( 0 );
			updateRate=upr;
		}
		OnSuspend();
	}catch( const char *p ){
		Die( p );
	}
	return 0;
}

int gxtkApp::InvokeOnResume(){
	try{
		OnResume();
		if( updateRate ){
			int upr=updateRate;
			updateRate=0;
			SetUpdateRate( upr );
		}
	}catch( const char *p ){
		Die( p );
	}
	return 0;
}

int gxtkApp::InvokeOnRender(){
	try{
		if( graphics->Validate() ){
			if( !created ){
				created=1;
				OnCreate();
			}
			graphics->BeginRender();
			OnRender();
			graphics->EndRender();
		}
	}catch( const char *p ){
		Die( p );
	}
	return 0;
}

gxtkGraphics *gxtkApp::GraphicsDevice(){
	return graphics;
}

gxtkInput *gxtkApp::InputDevice(){
	return input;
}

gxtkAudio *gxtkApp::AudioDevice(){
	return audio;
}

String gxtkApp::AppTitle(){
	return "<TODO>";
}

String gxtkApp::LoadState(){
	NSUserDefaults *prefs=[NSUserDefaults standardUserDefaults];
	NSString *nsstr=[prefs stringForKey:@"gxtkAppState"];
	if( nsstr ) return tostring( nsstr );
	return "";
}

int gxtkApp::SaveState( String state ){
	NSString *nsstr=tonsstr( state );
	NSUserDefaults *prefs=[NSUserDefaults standardUserDefaults];
	[prefs setObject:nsstr forKey:@"gxtkAppState"];
	return 0;
}

String gxtkApp::LoadString( String path ){
	if( path==L"" ) return L"";
//${TEXTFILES_BEGIN}
		else if( path==L"fonts/angel3.txt" ) return L"id,x,y,width,height,xoffset,yoffset,xadvance,page,chnl\r\n32,185,111,3,3,-1,26,9,0,15\r\n33,16,95,5,21,3,7,10,0,15\r\n34,123,111,10,9,1,6,12,0,15\r\n35,77,50,18,21,1,7,21,0,15\r\n36,0,0,16,27,1,6,17,0,15\r\n37,96,27,26,21,1,7,28,0,15\r\n38,186,23,20,21,0,7,19,0,15\r\n39,134,111,5,9,1,6,7,0,15\r\n40,47,0,10,27,1,6,12,0,15\r\n41,58,0,10,27,1,6,12,0,15\r\n42,42,113,14,14,1,6,17,0,15\r\n43,22,95,18,18,2,10,21,0,15\r\n44,77,113,7,10,1,22,9,0,15\r\n45,154,111,10,4,1,17,12,0,15\r\n46,148,111,5,6,2,22,9,0,15\r\n47,116,0,13,26,-1,6,12,0,15\r\n48,96,71,15,21,1,7,17,0,15\r\n49,203,67,13,21,2,7,17,0,15\r\n50,112,71,15,21,1,7,17,0,15\r\n51,0,73,15,21,1,7,17,0,15\r\n52,204,45,16,21,0,7,17,0,15\r\n53,32,73,15,21,1,7,17,0,15\r\n54,80,72,15,21,1,7,17,0,15\r\n55,48,73,15,21,1,7,17,0,15\r\n56,64,73,15,21,1,7,17,0,15\r\n57,16,73,15,21,1,7,17,0,15\r\n58,249,89,5,17,3,11,12,0,15\r\n59,247,23,7,21,2,11,12,0,15\r\n60,105,93,17,17,2,10,21,0,15\r\n61,105,111,17,9,2,14,21,0,15\r\n62,87,94,17,17,2,10,21,0,15\r\n63,217,67,13,21,1,7,14,0,15\r\n64,154,0,24,24,1,7,26,0,15\r\n65,123,27,20,21,-1,7,18,0,15\r\n66"
L",238,45,16,21,2,7,18,0,15\r\n67,96,49,18,21,0,7,18,0,15\r\n68,151,49,17,21,2,7,20,0,15\r\n69,159,71,14,21,2,7,16,0,15\r\n70,174,69,14,21,2,7,15,0,15\r\n71,207,23,19,21,0,7,20,0,15\r\n72,221,45,16,21,2,7,20,0,15\r\n73,0,95,9,21,1,7,11,0,15\r\n74,243,67,11,21,0,7,12,0,15\r\n75,115,49,17,21,2,7,18,0,15\r\n76,189,67,13,21,2,7,14,0,15\r\n77,20,51,18,21,2,7,22,0,15\r\n78,128,71,15,21,2,7,19,0,15\r\n79,144,27,20,21,0,7,20,0,15\r\n80,144,71,14,21,2,7,16,0,15\r\n81,95,0,20,26,0,7,20,0,15\r\n82,39,51,18,21,2,7,18,0,15\r\n83,187,45,16,21,1,7,18,0,15\r\n84,0,51,19,21,-1,7,16,0,15\r\n85,133,49,17,21,1,7,19,0,15\r\n86,165,25,20,21,-1,7,18,0,15\r\n87,69,28,26,21,0,7,26,0,15\r\n88,58,51,18,21,0,7,18,0,15\r\n89,227,23,19,21,-1,7,16,0,15\r\n90,169,47,17,21,1,7,18,0,15\r\n91,79,0,9,27,2,6,12,0,15\r\n92,130,0,13,26,0,6,12,0,15\r\n93,69,0,9,27,1,6,12,0,15\r\n94,57,113,19,12,1,7,21,0,15\r\n95,165,111,19,3,-1,28,17,0,15\r\n96,140,111,7,7,3,4,17,0,15\r\n97,188,91,15,17,0,11,16,0,15\r\n98,32,28,15,22,1,6,16,0,15\r\n99,204,89,14,17,0,11,14,0,15\r\n100,16,28,15,22,0,6,16,0,15\r\n101,140,93,15,17,0,11,15,"
L"0,15\r\n102,243,0,12,22,-1,6,9,0,15\r\n103,227,0,15,22,0,11,16,0,15\r\n104,48,28,14,22,1,6,16,0,15\r\n105,10,95,5,21,1,7,7,0,15\r\n106,144,0,9,26,-1,7,9,0,15\r\n107,211,0,15,22,1,6,15,0,15\r\n108,63,28,5,22,1,6,7,0,15\r\n109,41,95,23,17,2,11,25,0,15\r\n110,219,89,14,17,1,11,16,0,15\r\n111,123,93,16,17,0,11,16,0,15\r\n112,0,28,15,22,1,11,16,0,15\r\n113,195,0,15,22,0,11,16,0,15\r\n114,30,114,11,17,1,11,11,0,15\r\n115,234,89,14,17,0,11,14,0,15\r\n116,231,67,11,21,-1,7,10,0,15\r\n117,0,117,14,17,1,11,16,0,15\r\n118,156,93,15,17,0,11,15,0,15\r\n119,65,95,21,17,0,11,21,0,15\r\n120,172,93,15,17,0,11,15,0,15\r\n121,179,0,15,22,0,11,15,0,15\r\n122,15,117,14,17,0,11,14,0,15\r\n123,17,0,14,27,1,6,17,0,15\r\n124,89,0,5,27,4,6,12,0,15\r\n125,32,0,14,27,2,6,17,0,15\r\n126,85,113,19,9,1,14,21,0,15\r\n";
		else if( path==L"fonts/angel_verdana.txt" ) return L"id,x,y,width,height,xoffset,yoffset,xadvance,page,chnl\r\n32,185,111,3,3,-1,26,9,0,15\r\n33,16,95,5,21,3,7,10,0,15\r\n34,123,111,10,9,1,6,12,0,15\r\n35,77,50,18,21,1,7,21,0,15\r\n36,0,0,16,27,1,6,17,0,15\r\n37,96,27,26,21,1,7,28,0,15\r\n38,186,23,20,21,0,7,19,0,15\r\n39,134,111,5,9,1,6,7,0,15\r\n40,47,0,10,27,1,6,12,0,15\r\n41,58,0,10,27,1,6,12,0,15\r\n42,42,113,14,14,1,6,17,0,15\r\n43,22,95,18,18,2,10,21,0,15\r\n44,77,113,7,10,1,22,9,0,15\r\n45,154,111,10,4,1,17,12,0,15\r\n46,148,111,5,6,2,22,9,0,15\r\n47,116,0,13,26,-1,6,12,0,15\r\n48,96,71,15,21,1,7,17,0,15\r\n49,203,67,13,21,2,7,17,0,15\r\n50,112,71,15,21,1,7,17,0,15\r\n51,0,73,15,21,1,7,17,0,15\r\n52,204,45,16,21,0,7,17,0,15\r\n53,32,73,15,21,1,7,17,0,15\r\n54,80,72,15,21,1,7,17,0,15\r\n55,48,73,15,21,1,7,17,0,15\r\n56,64,73,15,21,1,7,17,0,15\r\n57,16,73,15,21,1,7,17,0,15\r\n58,249,89,5,17,3,11,12,0,15\r\n59,247,23,7,21,2,11,12,0,15\r\n60,105,93,17,17,2,10,21,0,15\r\n61,105,111,17,9,2,14,21,0,15\r\n62,87,94,17,17,2,10,21,0,15\r\n63,217,67,13,21,1,7,14,0,15\r\n64,154,0,24,24,1,7,26,0,15\r\n65,123,27,20,21,-1,7,18,0,15\r\n66"
L",238,45,16,21,2,7,18,0,15\r\n67,96,49,18,21,0,7,18,0,15\r\n68,151,49,17,21,2,7,20,0,15\r\n69,159,71,14,21,2,7,16,0,15\r\n70,174,69,14,21,2,7,15,0,15\r\n71,207,23,19,21,0,7,20,0,15\r\n72,221,45,16,21,2,7,20,0,15\r\n73,0,95,9,21,1,7,11,0,15\r\n74,243,67,11,21,0,7,12,0,15\r\n75,115,49,17,21,2,7,18,0,15\r\n76,189,67,13,21,2,7,14,0,15\r\n77,20,51,18,21,2,7,22,0,15\r\n78,128,71,15,21,2,7,19,0,15\r\n79,144,27,20,21,0,7,20,0,15\r\n80,144,71,14,21,2,7,16,0,15\r\n81,95,0,20,26,0,7,20,0,15\r\n82,39,51,18,21,2,7,18,0,15\r\n83,187,45,16,21,1,7,18,0,15\r\n84,0,51,19,21,-1,7,16,0,15\r\n85,133,49,17,21,1,7,19,0,15\r\n86,165,25,20,21,-1,7,18,0,15\r\n87,69,28,26,21,0,7,26,0,15\r\n88,58,51,18,21,0,7,18,0,15\r\n89,227,23,19,21,-1,7,16,0,15\r\n90,169,47,17,21,1,7,18,0,15\r\n91,79,0,9,27,2,6,12,0,15\r\n92,130,0,13,26,0,6,12,0,15\r\n93,69,0,9,27,1,6,12,0,15\r\n94,57,113,19,12,1,7,21,0,15\r\n95,165,111,19,3,-1,28,17,0,15\r\n96,140,111,7,7,3,4,17,0,15\r\n97,188,91,15,17,0,11,16,0,15\r\n98,32,28,15,22,1,6,16,0,15\r\n99,204,89,14,17,0,11,14,0,15\r\n100,16,28,15,22,0,6,16,0,15\r\n101,140,93,15,17,0,11,15,"
L"0,15\r\n102,243,0,12,22,-1,6,9,0,15\r\n103,227,0,15,22,0,11,16,0,15\r\n104,48,28,14,22,1,6,16,0,15\r\n105,10,95,5,21,1,7,7,0,15\r\n106,144,0,9,26,-1,7,9,0,15\r\n107,211,0,15,22,1,6,15,0,15\r\n108,63,28,5,22,1,6,7,0,15\r\n109,41,95,23,17,2,11,25,0,15\r\n110,219,89,14,17,1,11,16,0,15\r\n111,123,93,16,17,0,11,16,0,15\r\n112,0,28,15,22,1,11,16,0,15\r\n113,195,0,15,22,0,11,16,0,15\r\n114,30,114,11,17,1,11,11,0,15\r\n115,234,89,14,17,0,11,14,0,15\r\n116,231,67,11,21,-1,7,10,0,15\r\n117,0,117,14,17,1,11,16,0,15\r\n118,156,93,15,17,0,11,15,0,15\r\n119,65,95,21,17,0,11,21,0,15\r\n120,172,93,15,17,0,11,15,0,15\r\n121,179,0,15,22,0,11,15,0,15\r\n122,15,117,14,17,0,11,14,0,15\r\n123,17,0,14,27,1,6,17,0,15\r\n124,89,0,5,27,4,6,12,0,15\r\n125,32,0,14,27,2,6,17,0,15\r\n126,85,113,19,9,1,14,21,0,15\r\n\r\nfirst,second,amount\r\n39,65,-1\r\n45,65,-1\r\n45,74,-1\r\n45,84,-2\r\n45,86,-1\r\n45,87,-1\r\n45,88,-1\r\n45,89,-2\r\n45,118,-1\r\n45,120,-1\r\n45,121,-1\r\n45,122,-1\r\n46,44,-2\r\n46,45,-2\r\n65,45,-1\r\n65,84,-2\r\n65,86,-1\r\n65,87,-1\r\n65,89,-1\r\n65,118,-1\r\n65,121,-1\r\n66,84,-1\r\n67,45,-1\r\n68,44,-1\r\n68,46,-1\r\n"
L"68,84,-1\r\n70,44,-4\r\n70,46,-4\r\n70,58,-1\r\n70,63,1\r\n70,65,-1\r\n70,97,-1\r\n70,101,-1\r\n70,111,-1\r\n75,45,-1\r\n75,97,-1\r\n75,101,-1\r\n75,111,-1\r\n75,117,-1\r\n75,118,-1\r\n75,119,-1\r\n75,121,-1\r\n76,39,-2\r\n76,45,-2\r\n76,74,1\r\n76,84,-2\r\n76,86,-1\r\n76,87,-1\r\n76,89,-2\r\n76,118,-1\r\n76,121,-1\r\n79,84,-1\r\n80,44,-4\r\n80,46,-4\r\n80,65,-1\r\n80,97,-1\r\n80,101,-1\r\n80,111,-1\r\n82,45,-1\r\n82,84,-1\r\n82,97,-1\r\n82,101,-1\r\n82,111,-1\r\n82,121,-1\r\n84,44,-4\r\n84,45,-2\r\n84,46,-4\r\n84,58,-3\r\n84,63,1\r\n84,65,-2\r\n84,67,-1\r\n84,71,-1\r\n84,79,-1\r\n84,84,-1\r\n84,97,-3\r\n84,99,-3\r\n84,101,-3\r\n84,103,-3\r\n84,111,-3\r\n84,114,-3\r\n84,115,-2\r\n84,117,-3\r\n84,118,-3\r\n84,119,-3\r\n84,121,-3\r\n84,122,-2\r\n86,44,-4\r\n86,45,-1\r\n86,46,-4\r\n86,58,-1\r\n86,65,-1\r\n86,97,-1\r\n86,101,-1\r\n86,111,-1\r\n86,117,-1\r\n86,121,-1\r\n87,44,-4\r\n87,45,-1\r\n87,46,-3\r\n87,58,-1\r\n87,65,-1\r\n87,97,-1\r\n87,101,-1\r\n87,111,-1\r\n87,114,-1\r\n87,117,-1\r\n87,121,-1\r\n88,45,-1\r\n88,97,-1\r\n88,101,-1\r\n88,111,-1\r\n88,121,-1\r\n89,44,-4\r\n89,45,-2\r\n89,46,-4\r\n89,58,-3\r\n89,65,-1\r\n89,97,-2\r\n89,100,-2\r\n89,101,-2\r\n89,103,-2\r\n89,109,-1\r\n89,110,-1\r\n89,11"
L"1,-2\r\n89,112,-1\r\n89,113,-2\r\n89,114,-1\r\n89,115,-1\r\n89,117,-1\r\n89,118,-1\r\n90,45,-1\r\n90,97,-1\r\n90,101,-1\r\n90,111,-1\r\n90,119,-1\r\n90,121,-1\r\n99,84,-1\r\n101,84,-2\r\n102,34,1\r\n102,39,1\r\n102,41,1\r\n102,42,1\r\n102,44,-2\r\n102,45,-1\r\n102,46,-2\r\n102,63,1\r\n102,92,1\r\n102,93,1\r\n102,125,1\r\n107,45,-1\r\n114,44,-4\r\n114,46,-4\r\n116,45,-1\r\n118,44,-2\r\n118,45,-1\r\n118,46,-2\r\n118,97,-1\r\n119,44,-1\r\n119,46,-1\r\n120,45,-1\r\n121,44,-2\r\n121,45,-1\r\n121,46,-2\r\n121,97,-1\r\n";
		else if( path==L"fonts/simpletextboxhtml.txt" ) return L"For the desktop browsers, the following techniques generated the <b>highest frame rates</b>:\n\nSnapping positions to integer values rather than using floating point values. JavaScript supports several different ways to snap, such as \"parseInt\" and \"| 0\". We would expect using the <i>or</i> operator to be faster than a <i>function</i> call, but have not yet tested this. Note that snapping could generate visible artifacts, particularly at very low resolutions.\n\nUsing the <i>DIV</i> tag with background image instead of <i>IMG</i> tag. The difference is particularly noticeable when using individual sprites instead of sprite sheets, as changing the IMG's src property is very slow.\n\nMany individual sprites are faster than sprite sheets in nearly every case. The exceptions were the fully hardware accelerated browsers on Windows, where sprite sheets offer a 5-10% advantage. Unless you are only targeting those browsers, we recommend staying with individual sprites. For browsers with the CANVAS tag, we expect an optimum"
L" balance of download and draw performance is to download sprite sheets but then use CANVAS to cut them up.\n\nUpdate DOM elements rather than using innerHTML to load the parent. For several browsers, Firefox 4, Chrome 10 and 11, and IE9, innerHTML was faster, particularly for rotated sprites. However, for those browsers CANVAS tag performance was far superior to DOM manipulation, so update and switch to CANVAS when possible.\n\nFor axis-aligned images with integer position values, the CANVAS tag was superior across all desktop browsers. This had not been the case with floating point values -- and indicates software blitting in their render pipelines -- but if you are snapping and drawing axis aligned images, use CANVAS when you are able to.\n\nFor browsers that support 3D transforms -- we are currently testing on Chrome and Safari -- using translate3d(x,y,z) is equal to or faster than translate(x,y).\n\nOn desktops, using CSS transitions for motion or CSS keyframes for animations were slower than simply using JavaScr"
L"ipt for these tasks. Worse, they often generated noisy framerates, so they are not a good solution for games in desktop browsers.\n";
		else if( path==L"fonts/simpletextboxtext.txt" ) return L"For the desktop browsers, the following techniques generated the highest frame rates:\n\nSnapping positions to integer values rather than using floating point values. JavaScript supports several different ways to snap, such as \"parseInt\" and \"| 0\". We would expect using the or operator to be faster than a function call, but have not yet tested this. Note that snapping could generate visible artifacts, particularly at very low resolutions.\n\nUsing the DIV tag with background image instead of IMG tag. The difference is particularly noticeable when using individual sprites instead of sprite sheets, as changing the IMG's src property is very slow.\n\nMany individual sprites are faster than sprite sheets in nearly every case. The exceptions were the fully hardware accelerated browsers on Windows, where sprite sheets offer a 5-10% advantage. Unless you are only targeting those browsers, we recommend staying with individual sprites. For browsers with the CANVAS tag, we expect an optimum balance of download and draw perfo"
L"rmance is to download sprite sheets but then use CANVAS to cut them up.\n\nUpdate DOM elements rather than using innerHTML to load the parent. For several browsers, Firefox 4, Chrome 10 and 11, and IE9, innerHTML was faster, particularly for rotated sprites. However, for those browsers CANVAS tag performance was far superior to DOM manipulation, so update and switch to CANVAS when possible.\n\nFor axis-aligned images with integer position values, the CANVAS tag was superior across all desktop browsers. This had not been the case with floating point values -- and indicates software blitting in their render pipelines -- but if you are snapping and drawing axis aligned images, use CANVAS when you are able to.\n\nFor browsers that support 3D transforms -- we are currently testing on Chrome and Safari -- using translate3d(x,y,z) is equal to or faster than translate(x,y).\n\nOn desktops, using CSS transitions for motion or CSS keyframes for animations were slower than simply using JavaScript for these tasks. Worse, they of"
L"ten generated noisy framerates, so they are not a good solution for games in desktop browsers.\n";
		return L"";

//${TEXTFILES_END}
}

int gxtkApp::SetUpdateRate( int hertz ){

	if( updateTimer ){
		//Kill acceler/timer
		//
#if ACCELEROMETER_ENABLED
		UIAccelerometer *acc=[UIAccelerometer sharedAccelerometer];
		UIApplication *app=[UIApplication sharedApplication];
		[acc setUpdateInterval:0.0];
		[acc setDelegate:0];
		[app setIdleTimerDisabled:NO];
#endif
		[updateTimer invalidate];
		updateTimer=0;
	}

	updateRate=hertz;

	if( updateRate ){
		//These added to runloop in order? ie: updates accel before timer?
		//
#if ACCELEROMETER_ENABLED
		UIAccelerometer *acc=[UIAccelerometer sharedAccelerometer];
		UIApplication *app=[UIApplication sharedApplication];
		[acc setUpdateInterval:1.0/updateRate];
		[acc setDelegate:[app delegate]];
		[app setIdleTimerDisabled:YES];
#endif
		updateTimer=[NSTimer 
		scheduledTimerWithTimeInterval:(NSTimeInterval)(1.0/updateRate)
		target:appDelegate
		selector:@selector(updateTimerFired) userInfo:nil repeats:TRUE];
	}
	return 0;
}

int gxtkApp::MilliSecs(){
	uint64_t nanos=mach_absolute_time()-startTime;
	nanos*=timeInfo.numer;
	nanos/=timeInfo.denom;
	return nanos/1000000L;
}

int gxtkApp::Loading(){
	return 0;
}

int gxtkApp::OnCreate(){
	return 0;
}

int gxtkApp::OnUpdate(){
	return 0;
}

int gxtkApp::OnSuspend(){
	return 0;
}

int gxtkApp::OnResume(){
	return 0;
}

int gxtkApp::OnRender(){
	return 0;
}

int gxtkApp::OnLoading(){
	return 0;
}

// ***** gxtkInput *****

gxtkInput::gxtkInput(){
	memset( keyStates,0,sizeof(keyStates) );
	memset( touches,0,sizeof(touches) );
	memset( touchX,0,sizeof(touchX) );
	memset( touchY,0,sizeof(touchY) );
	accelX=accelY=accelZ=0;
}

void gxtkInput::BeginUpdate(){
}

void gxtkInput::EndUpdate(){
	for( int i=0;i<512;++i ){
		keyStates[i]&=0x100;
	}
}

void gxtkInput::OnEvent( UIEvent *event ){
	if( [event type]==UIEventTypeTouches ){
	
		UIView *view=app->appDelegate->view;

		float scaleFactor=1.0f;
		if( [view respondsToSelector:@selector(contentScaleFactor)] ){
			scaleFactor=[view contentScaleFactor];
		}

		for( UITouch *touch in [event allTouches] ){
		
			int pid;
			for( pid=0;pid<32 && touches[pid]!=touch;++pid ) {}

			switch( [touch phase] ){
			case UITouchPhaseBegan:
				if( pid!=32 ){ pid=32;break; }
				for( pid=0;pid<32 && touches[pid];++pid ){}
				if( pid==32 ) break;
				touches[pid]=touch;
				keyStates[KEY_TOUCH0+pid]=0x101;
				break;
			case UITouchPhaseEnded:
			case UITouchPhaseCancelled:
				if( pid==32 ) break;
				touches[pid]=0;
				keyStates[KEY_TOUCH0+pid]=0;
				break;
			}
			if( pid==32 ){
				printf( "***** GXTK Touch Error *****\n" );fflush( stdout );
				continue;
			}
			
			CGPoint p=[touch locationInView:view];
			p.x*=scaleFactor;
			p.y*=scaleFactor;
			
			touchX[pid]=p.x;
			touchY[pid]=p.y;
		}
	}
}

void gxtkInput::OnAcceleration( UIAcceleration *accel ){
	accelX=accel.x;
	accelY=-accel.y;
	accelZ=-accel.z;
}

int gxtkInput::KeyDown( int key ){
	if( key>0 && key<512 ){
		if( key==KEY_LMB ) key=KEY_TOUCH0;
		return keyStates[key] >> 8;
	}
	return 0;
}

int gxtkInput::KeyHit( int key ){
	if( key>0 && key<512 ){
		if( key==KEY_LMB ) key=KEY_TOUCH0;
		return keyStates[key] & 0xff;
	}
	return 0;
}

int gxtkInput::GetChar(){
	return 0;
}
	
float gxtkInput::MouseX(){
	return touchX[0];
}

float gxtkInput::MouseY(){
	return touchY[0];
}

float gxtkInput::JoyX( int index ){
	return 0;
}

float gxtkInput::JoyY( int index ){
	return 0;
}

float gxtkInput::JoyZ( int index ){
	return 0;
}

float gxtkInput::TouchX( int index ){
	return touchX[index];
}

float gxtkInput::TouchY( int index ){
	return touchY[index];
}

float gxtkInput::AccelX(){
	return accelX;
}

float gxtkInput::AccelY(){
	return accelY;
}

float gxtkInput::AccelZ(){
	return accelZ;
}

//***** gxtkAudio *****
static void CheckAL(){
	ALenum err=alGetError();
	if( err!=AL_NO_ERROR ){
		printf( "AL Error:%i\n",err );
		fflush( stdout );
	}
}

gxtkAudio::gxtkAudio():nextSource(0){
	alDistanceModel( AL_NONE );
	memset( al_sources,0,sizeof(al_sources) );
}

int gxtkAudio::AL_Source( int channel ){
	if( channel<0 || channel>=MAX_CHANNELS ) return 0;
	if( !al_sources[channel] ) alGenSources( 1,&al_sources[channel] );
	return al_sources[channel];
}

gxtkSample *gxtkAudio::LoadSample( String path ){
	return new gxtkSample( path );
}

int gxtkAudio::DestroySample( gxtkSample *sample ){
	alDeleteBuffers( 1,&sample->al_buffer );
	return 0;
}

int gxtkAudio::PlaySample( gxtkSample *sample,int channel,int flags ){
	ALint source=AL_Source( channel );
	alSourceStop( source );
	alSourcei( source,AL_BUFFER,sample->al_buffer );
	alSourcei( source,AL_LOOPING,flags ? 1 : 0 );
	alSourcePlay( source );
	return 0;
}

int gxtkAudio::StopChannel( int channel ){
	alSourceStop( AL_Source( channel ) );
	return 0;
}

int gxtkAudio::ChannelState( int channel ){
	int state=0;
	alGetSourcei( AL_Source( channel ),AL_SOURCE_STATE,&state );
	return (state==AL_PLAYING) ? 1 : 0;
}

int gxtkAudio::SetVolume( int channel,float volume ){
	alSourcef( AL_Source( channel ),AL_GAIN,volume );
	return 0;
}

int gxtkAudio::SetPan( int channel,float pan ){
	alSource3f( AL_Source( channel ),AL_POSITION,pan,0,0 );
	return 0;
}

int gxtkAudio::SetRate( int channel,float rate ){
	alSourcef( AL_Source( channel ),AL_PITCH,rate );
	return 0;
}

//***** gxtkSample *****

static const char *ReadTag( FILE *f ){
	static char buf[8];
	if( fread( buf,4,1,f )!=1 ) return "";
	buf[4]=0;
	return buf;
}

static int ReadInt( FILE *f ){
	unsigned char buf[4];
	if( fread( buf,4,1,f )!=1 ) return -1;
	return (buf[3]<<24) | (buf[2]<<16) | (buf[1]<<8) | buf[0];
}

static int ReadShort( FILE *f ){
	unsigned char buf[2];
	if( fread( buf,2,1,f )!=1 ) return -1;
	return (buf[1]<<8) | buf[0];
}

static void SkipBytes( int n,FILE *f ){
	char *p=(char*)malloc( n );
	fread( p,n,1,f );
	free(p);
}

gxtkSample::gxtkSample( String path ):al_buffer(0){

	NSString *rpath=[[NSBundle mainBundle] pathForResource:tonsstr(path) ofType:nil inDirectory:@"data"];
	if( FILE *f=fopen([rpath cStringUsingEncoding:1],"rb" ) ){

	//if( FILE *f=fopen( (T("data/")+path).ToCString<char>(),"rb" ) ){
	
	
		if( !strcmp( ReadTag( f ),"RIFF" ) ){
			int len=ReadInt( f )-8;
			if( !strcmp( ReadTag( f ),"WAVE" ) ){
				if( !strcmp( ReadTag( f ),"fmt " ) ){
					int len2=ReadInt( f );
					int comp=ReadShort( f );
					if( comp==1 ){
						int chans=ReadShort( f );
						int hertz=ReadInt( f );
						int bytespersec=ReadInt( f );
						int pad=ReadShort( f );
						int bits=ReadShort( f );
						int format=0;
						if( bits==8 && chans==1 ){
							format=AL_FORMAT_MONO8;
						}else if( bits==8 && chans==2 ){
							format=AL_FORMAT_STEREO8;
						}else if( bits==16 && chans==1 ){
							format=AL_FORMAT_MONO16;
						}else if( bits==16 && chans==2 ){
							format=AL_FORMAT_STEREO16;
						}
						if( format ){
							if( len2>16 ) SkipBytes( len2-16,f );
							for(;;){
								const char *p=ReadTag( f );
								if( feof( f ) ) break;
								int size=ReadInt( f );
								if( strcmp( p,"data" ) ){
									SkipBytes( size,f );
									continue;
								}
								char *data=(char*)malloc( size );
								if( fread( data,size,1,f )==1 ){
									alGenBuffers( 1,&al_buffer );
									alBufferData( al_buffer,format,data,size,hertz );
								}
								free( data );
								break;
							}
						}
					}
				}
			}
		}
		fclose( f );
	}
}

// ***** Ok, we have to implement app delegate ourselves *****

int gc_millis;

void gc_collect(){
	gc_millis=app->MilliSecs();
	::gc_mark();
	::gc_sweep();
	gc_millis=app->MilliSecs()-gc_millis;
}

bool app_suspended;

@implementation MonkeyAppDelegate

@synthesize window;
@synthesize view;
@synthesize viewController;

-(void)drawView:(MonkeyView*)aview{
	if( app ){
		app->InvokeOnRender();
		gc_collect();
	}else{
		exit( 0 );
	}
}

-(void)onEvent:(UIEvent*)event{
	if( app ){
		app->input->OnEvent( event );
	}
}

-(void)accelerometer:(UIAccelerometer*)accelerometer didAccelerate:(UIAcceleration *)acceleration{
	if( app ){
		app->input->OnAcceleration( acceleration );
	}
}

-(BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary*)launchOptions{
    
	bb_std_main( 0,0 );
	
	if( !app ) exit( 0 );

	[[UIApplication sharedApplication] setStatusBarHidden:YES animated:NO];
	
	[viewController.view setFrame:[[UIScreen mainScreen] applicationFrame]];

    [window addSubview:viewController.view];

    [window makeKeyAndVisible];

	ALCdevice *alcDevice=alcOpenDevice( 0 );
	if( !alcDevice ){
		puts( "alcOpenDevice failed" );
		exit( -1 );
	}

	ALCcontext *alcContext=alcCreateContext( alcDevice,0 );
	if( !alcContext ){
		puts( "alcCreateContext failed" );
		exit( -1 );
	}

	if( !alcMakeContextCurrent( alcContext ) ){
		puts( "alcmakeContextCurrent failed" );
		exit( -1 );
	}

	return YES;
}

-(void)applicationWillResignActive:(UIApplication *)application{
	if( app_suspended ) return;
	
	app_suspended=true;
	app->InvokeOnSuspend();
}

-(void)applicationDidBecomeActive:(UIApplication *)application{
	if( !app_suspended ) return;
	
	app->InvokeOnResume();
	app_suspended=false;
}

-(void)applicationDidEnterBackground:(UIApplication *)application{
}

-(void)applicationWillTerminate:(UIApplication *)application{
}

-(void)updateTimerFired{
	if( app_suspended || !app->updateRate ) return;
	
	app->InvokeOnUpdate();
	app->InvokeOnRender();
	gc_collect();
}

-(void)dealloc{
	[window release];
	[view release];
	[super dealloc];
}

@end
class bb_app_App;
class bb_application_Application;
class bb_app_AppDevice;
class bb_graphics_GraphicsContext;
class bb_fader_Fader;
class bb_list_List;
class bb_scene_Scene;
class bb_boxes_StringObject;
class bb_map_Map;
class bb_map_StringMap;
class bb_faderbrightness_FaderBrightness;
class bb_list_Node;
class bb_soundmanager_SoundManager;
class bb_resource_Resource;
class bb_audio_Sound;
class bb_map_Node;
class bb_gamefonts_GameFonts;
class bb_angelfont_AngelFont;
class bb_kernpair_KernPair;
class bb_char_Char;
class bb_graphics_Image;
class bb_graphics_Frame;
class bb_scenelogo_SceneLogo;
class bb_scenetitle_SceneTitle;
class bb_scenegame_SceneGame;
class bb_list_Enumerator;
class bb_gameobject_GameObject;
class bb_base_Base;
class bb_station_Station;
class bb_bullet_Bullet;
class bb_explosion_Explosion;
class bb_asteroid_Asteroid;
class bb_powerup_PowerUp;
class bb_collision_Collision;
class bb_app_App : public Object{
	public:
	bb_app_App();
	bb_app_App* bbnew();
	virtual int bbOnCreate();
	virtual int bbOnUpdate();
	virtual int bbOnSuspend();
	virtual int bbOnResume();
	virtual int bbOnRender();
	virtual int bbOnLoading();
	void mark();
};
class bb_application_Application : public bb_app_App{
	public:
	bb_list_List* bbfaders;
	bb_map_StringMap* bbscenes;
	bb_scene_Scene* bbcurrentScene;
	int bbstate;
	bool bbloading;
	bb_scene_Scene* bbnextScene;
	bb_application_Application();
	static bb_application_Application* bbapplication;
	bb_application_Application* bbnew();
	static bb_application_Application* bbGetInstance();
	virtual void bbInit();
	virtual void bbAddFader(bb_fader_Fader* bbf);
	virtual void bbAddScene(String bbsceneName,bb_scene_Scene* bbscene);
	virtual void bbRun();
	virtual int bbOnCreate();
	virtual void bbSetState(int bbst);
	virtual int bbOnUpdate();
	virtual int bbOnRender();
	virtual int bbOnLoading();
	virtual int bbOnResume();
	virtual int bbOnSuspend();
	virtual void bbSetNextScene(String bbsceneName);
	void mark();
};
class bb_app_AppDevice : public gxtkApp{
	public:
	bb_app_App* bbapp;
	bb_app_AppDevice();
	bb_app_AppDevice* bbnew(bb_app_App* bbapp);
	bb_app_AppDevice* bbnew2();
	virtual int OnCreate();
	virtual int OnUpdate();
	virtual int OnSuspend();
	virtual int OnResume();
	virtual int OnRender();
	virtual int OnLoading();
	void mark();
};
class bb_graphics_GraphicsContext : public Object{
	public:
	gxtkGraphics* bbdevice;
	int bbmatrixSp;
	float bbix;
	float bbiy;
	float bbjx;
	float bbjy;
	float bbtx;
	float bbty;
	int bbtformed;
	int bbmatDirty;
	float bbcolor_r;
	float bbcolor_g;
	float bbcolor_b;
	float bbalpha;
	int bbblend;
	float bbscissor_x;
	float bbscissor_y;
	float bbscissor_width;
	float bbscissor_height;
	bb_graphics_Image* bbfont;
	bb_graphics_Image* bbdefaultFont;
	int bbfirstChar;
	Array<float > bbmatrixStack;
	bb_graphics_GraphicsContext();
	bb_graphics_GraphicsContext* bbnew(gxtkGraphics* bbdevice);
	bb_graphics_GraphicsContext* bbnew2();
	void mark();
};
extern bb_graphics_GraphicsContext* bb_graphics_context;
int bb_graphics_SetGraphicsContext(bb_graphics_GraphicsContext* bbgc);
extern gxtkInput* bb_input_device;
int bb_input_SetInputDevice(gxtkInput* bbdev);
extern gxtkAudio* bb_audio_device;
int bb_audio_SetAudioDevice(gxtkAudio* bbdev);
extern bb_app_AppDevice* bb_app_device;
class bb_fader_Fader : public Object{
	public:
	float bbspeed;
	int bbfadeState;
	float bbposition;
	bb_fader_Fader();
	bb_fader_Fader* bbnew();
	virtual void bbFadeIn();
	virtual bool bbIsFading();
	virtual void bbOnUpdate();
	virtual void bbOnRender();
	virtual void bbFadeOut();
	void mark();
};
class bb_list_List : public Object{
	public:
	bb_list_Node* bb_head;
	bb_list_List();
	bb_list_List* bbnew();
	virtual bb_list_Node* bbAddLast(Object* bbdata);
	virtual bb_list_Enumerator* bbObjectEnumerator();
	virtual int bbClear();
	virtual int bbEquals(Object* bblhs,Object* bbrhs);
	virtual int bbRemoveEach(Object* bbvalue);
	virtual int bbRemove(Object* bbvalue);
	void mark();
};
class bb_scene_Scene : public Object{
	public:
	String bbname;
	bb_scene_Scene();
	bb_scene_Scene* bbnew();
	virtual void bbOnEnter();
	virtual void bbOnUpdate();
	virtual void bbOnLeave();
	virtual void bbOnRender();
	void mark();
};
class bb_boxes_StringObject : public Object{
	public:
	String bbvalue;
	bb_boxes_StringObject();
	bb_boxes_StringObject* bbnew(int bbvalue);
	bb_boxes_StringObject* bbnew2(float bbvalue);
	bb_boxes_StringObject* bbnew3(String bbvalue);
	bb_boxes_StringObject* bbnew4();
	void mark();
};
class bb_map_Map : public Object{
	public:
	bb_map_Node* bbroot;
	bb_map_Map();
	bb_map_Map* bbnew();
	virtual int bbCompare(Object* bblhs,Object* bbrhs);
	virtual bb_map_Node* bbFindNode(Object* bbkey);
	virtual Object* bbGet(Object* bbkey);
	virtual Object* bbValueForKey(Object* bbkey);
	virtual int bbRotateLeft(bb_map_Node* bbnode);
	virtual int bbRotateRight(bb_map_Node* bbnode);
	virtual int bbInsertFixup(bb_map_Node* bbnode);
	virtual int bbSet(Object* bbkey,Object* bbvalue);
	virtual int bbInsert(Object* bbkey,Object* bbvalue);
	virtual bool bbIsEmpty();
	virtual bool bbContains(Object* bbkey);
	void mark();
};
class bb_map_StringMap : public bb_map_Map{
	public:
	bb_map_StringMap();
	bb_map_StringMap* bbnew();
	virtual int bbCompare(Object* bblhs,Object* bbrhs);
	void mark();
};
class bb_faderbrightness_FaderBrightness : public bb_fader_Fader{
	public:
	bb_faderbrightness_FaderBrightness();
	bb_faderbrightness_FaderBrightness* bbnew();
	virtual void bbOnRender();
	void mark();
};
class bb_list_Node : public Object{
	public:
	bb_list_Node* bb_succ;
	bb_list_Node* bb_pred;
	Object* bb_data;
	bb_list_Node();
	bb_list_Node* bbnew();
	bb_list_Node* bbnew2(bb_list_Node* bbsucc,bb_list_Node* bbpred,Object* bbdata);
	virtual int bbRemove();
	void mark();
};
class bb_soundmanager_SoundManager : public Object{
	public:
	bb_soundmanager_SoundManager();
	static bb_audio_Sound* bbbackgroundMusic;
	static Array<bb_audio_Sound* > bbsfx;
	static void bbLoadSounds();
	static void bbPlayMusic();
	static void bbPlay(int bbsound);
	void mark();
};
class bb_resource_Resource : public Object{
	public:
	bb_list_Node* bbnode;
	int bbrefs;
	bb_resource_Resource();
	bb_resource_Resource* bbnew();
	virtual int bbRegister(String bbtype);
	virtual int bbRetain();
	void mark();
};
class bb_audio_Sound : public bb_resource_Resource{
	public:
	gxtkSample* bbsample;
	bb_audio_Sound();
	bb_audio_Sound* bbnew(gxtkSample* bbsample);
	bb_audio_Sound* bbnew2();
	void mark();
};
extern bb_map_StringMap* bb_resource_resources;
class bb_map_Node : public Object{
	public:
	Object* bbkey;
	bb_map_Node* bbright;
	bb_map_Node* bbleft;
	Object* bbvalue;
	int bbcolor;
	bb_map_Node* bbparent;
	bb_map_Node();
	bb_map_Node* bbnew(Object* bbkey,Object* bbvalue,int bbcolor,bb_map_Node* bbparent);
	bb_map_Node* bbnew2();
	void mark();
};
bb_audio_Sound* bb_audio_LoadSound(String bbpath);
class bb_gamefonts_GameFonts : public Object{
	public:
	bb_gamefonts_GameFonts();
	static bb_angelfont_AngelFont* bbnormal;
	static void bbLoad();
	void mark();
};
class bb_angelfont_AngelFont : public Object{
	public:
	String bbiniText;
	bb_map_StringMap* bbkernPairs;
	Array<bb_char_Char* > bbchars;
	int bbheight;
	int bbheightOffset;
	bb_graphics_Image* bbimage;
	String bbname;
	int bbxOffset;
	bool bbuseKerning;
	bb_angelfont_AngelFont();
	static String bberror;
	static bb_angelfont_AngelFont* bbcurrent;
	virtual void bbLoadFont(String bburl);
	static bb_map_StringMap* bb_list;
	bb_angelfont_AngelFont* bbnew(String bburl);
	bb_angelfont_AngelFont* bbnew2();
	virtual void bbDrawText(String bbtxt,int bbx,int bby);
	virtual int bbTextWidth(String bbtxt);
	virtual void bbDrawText2(String bbtxt,int bbx,int bby,int bbalign);
	void mark();
};
String bb_app_LoadString(String bbpath);
class bb_kernpair_KernPair : public Object{
	public:
	String bbfirst;
	String bbsecond;
	int bbamount;
	bb_kernpair_KernPair();
	bb_kernpair_KernPair* bbnew(int bbfirst,int bbsecond,int bbamount);
	bb_kernpair_KernPair* bbnew2();
	void mark();
};
class bb_char_Char : public Object{
	public:
	int bbx;
	int bby;
	int bbwidth;
	int bbheight;
	int bbxOffset;
	int bbyOffset;
	int bbxAdvance;
	bb_char_Char();
	bb_char_Char* bbnew(int bbx,int bby,int bbw,int bbh,int bbxoff,int bbyoff,int bbxadv);
	bb_char_Char* bbnew2();
	virtual int bbDraw(bb_graphics_Image* bbfontImage,int bblinex,int bbliney);
	void mark();
};
class bb_graphics_Image : public bb_resource_Resource{
	public:
	gxtkSurface* bbsurface;
	int bbwidth;
	int bbheight;
	int bbflags;
	Array<bb_graphics_Frame* > bbframes;
	float bbtx;
	float bbty;
	bb_graphics_Image* bbsource;
	bb_graphics_Image();
	static int bbDefaultFlags;
	virtual int bbSetHandle(float bbtx,float bbty);
	bb_graphics_Image* bbnew(String bbpath,int bbnframes,int bbiflags);
	bb_graphics_Image* bbnew2(int bbx,int bby,int bbiwidth,int bbiheight,int bbnframes,int bbiflags,bb_graphics_Image* bbsource);
	bb_graphics_Image* bbnew3();
	virtual bb_graphics_Image* bbGrabImage(int bbx,int bby,int bbwidth,int bbheight,int bbframes,int bbflags);
	virtual int bbWidth();
	virtual int bbLoaded();
	virtual int bbHeight();
	virtual int bbFrames();
	void mark();
};
class bb_graphics_Frame : public Object{
	public:
	int bbx;
	int bby;
	bb_graphics_Frame();
	bb_graphics_Frame* bbnew(int bbx,int bby);
	bb_graphics_Frame* bbnew2();
	void mark();
};
bb_graphics_Image* bb_graphics_LoadImage(String bbpath,int bbframeCount,int bbflags);
bb_graphics_Image* bb_graphics_LoadImage2(String bbpath,int bbframeWidth,int bbframeHeight,int bbframeCount,int bbflags);
class bb_scenelogo_SceneLogo : public bb_scene_Scene{
	public:
	bb_graphics_Image* bblogo;
	int bbtimer;
	bb_scenelogo_SceneLogo();
	bb_scenelogo_SceneLogo* bbnew();
	virtual void bbOnEnter();
	virtual void bbOnUpdate();
	virtual void bbOnRender();
	virtual void bbOnLeave();
	void mark();
};
class bb_scenetitle_SceneTitle : public bb_scene_Scene{
	public:
	bb_graphics_Image* bbtitlepic;
	bb_scenetitle_SceneTitle();
	bb_scenetitle_SceneTitle* bbnew();
	virtual void bbOnEnter();
	virtual void bbOnUpdate();
	virtual void bbOnRender();
	virtual void bbOnLeave();
	void mark();
};
class bb_scenegame_SceneGame : public bb_scene_Scene{
	public:
	int bblevel;
	int bbscore;
	int bbstate;
	Array<bb_graphics_Image* > bbbackground;
	int bblaunchDelay;
	int bbtimer;
	int bblaunchAmount;
	float bblaunchDx;
	float bblaunchDy;
	int bblaunchCount;
	bb_scenegame_SceneGame();
	bb_scenegame_SceneGame* bbnew();
	virtual void bbInitGame();
	virtual void bbInitLevel(int bbl);
	virtual void bbOnEnter();
	virtual void bbClearLevel();
	virtual void bbCheckCollision();
	virtual void bbOnUpdate();
	virtual void bbRenderStateMessage(String bbt,bool bbblink);
	virtual String bbGetScore();
	virtual void bbRenderScore();
	virtual void bbOnRender();
	virtual void bbOnLeave();
	void mark();
};
class bb_list_Enumerator : public Object{
	public:
	bb_list_List* bb_list;
	bb_list_Node* bb_curr;
	bb_list_Enumerator();
	bb_list_Enumerator* bbnew(bb_list_List* bblist);
	bb_list_Enumerator* bbnew2();
	virtual bool bbHasNext();
	virtual Object* bbNextObject();
	void mark();
};
int bb_Main();
int bb_graphics_SetMatrix(float bbix,float bbiy,float bbjx,float bbjy,float bbtx,float bbty);
int bb_graphics_SetMatrix2(Array<float > bbm);
int bb_graphics_SetColor(float bbr,float bbg,float bbb);
int bb_graphics_SetAlpha(float bbalpha);
int bb_graphics_SetBlend(int bbblend);
int bb_graphics_DeviceWidth();
int bb_graphics_DeviceHeight();
int bb_graphics_SetScissor(float bbx,float bby,float bbwidth,float bbheight);
int bb_graphics_BeginRender();
int bb_graphics_EndRender();
class bb_gameobject_GameObject : public Object{
	public:
	float bbx;
	float bby;
	bb_gameobject_GameObject();
	static void bbInit();
	static bb_list_List* bblist;
	bb_gameobject_GameObject* bbnew();
	virtual bool bbIsDestroyed();
	virtual void bbDestroy();
	virtual void bbOnUpdate();
	static void bbUpdateAll();
	virtual void bbOnRender();
	static void bbRenderAll();
	void mark();
};
class bb_base_Base : public bb_gameobject_GameObject{
	public:
	bb_base_Base();
	static bb_graphics_Image* bbimg;
	static void bbInit();
	bb_base_Base* bbnew();
	static bb_base_Base* bbCreate();
	virtual void bbOnRender();
	virtual void bbOnUpdate();
	virtual void bbDestroy();
	void mark();
};
class bb_station_Station : public bb_gameobject_GameObject{
	public:
	bool bbdestroyed;
	int bbtype;
	bb_station_Station();
	static Array<bb_graphics_Image* > bbimg;
	static float bbcollisionRadius;
	static void bbInit();
	bb_station_Station* bbnew();
	static void bbCreate(int bbx,int bby,int bbtype);
	virtual bool bbIsDestroyed();
	virtual void bbOnRender();
	virtual void bbOnUpdate();
	virtual void bbDestroy();
	void mark();
};
class bb_bullet_Bullet : public bb_gameobject_GameObject{
	public:
	int bbstate;
	int bbsteps;
	float bbdx;
	float bbdy;
	bb_bullet_Bullet();
	static bb_graphics_Image* bbimg;
	static int bbcollisionRadius;
	static int bbbullets;
	static int bbbulletsInUse;
	static void bbInit();
	virtual void bbDestroy();
	bb_bullet_Bullet* bbnew();
	static int bbLAUNCH_X;
	static int bbLAUNCH_Y;
	static int bbSPEED;
	static bb_bullet_Bullet* bbCreate(int bbdx,int bbdy);
	virtual bool bbIsDestroyed();
	virtual void bbOnUpdate();
	virtual void bbOnRender();
	static void bbPowerUp();
	void mark();
};
class bb_explosion_Explosion : public bb_gameobject_GameObject{
	public:
	float bbframe;
	int bbtyp;
	bb_explosion_Explosion();
	static Array<bb_graphics_Image* > bbimg;
	static float bbcollisionRadius;
	static void bbInit();
	virtual void bbDestroy();
	virtual bool bbIsDestroyed();
	bb_explosion_Explosion* bbnew();
	static bb_explosion_Explosion* bbCreate(int bbx,int bby,int bbtyp);
	virtual void bbOnUpdate();
	virtual void bbOnRender();
	void mark();
};
class bb_asteroid_Asteroid : public bb_gameobject_GameObject{
	public:
	int bbstate;
	float bbsx;
	float bbsy;
	float bbdx;
	float bbdy;
	float bbframe;
	bb_asteroid_Asteroid();
	static bb_graphics_Image* bbimg;
	static float bbcollisionRadius;
	static void bbInit();
	virtual bool bbIsDestroyed();
	bb_asteroid_Asteroid* bbnew();
	static int bbcount;
	static bb_asteroid_Asteroid* bbCreate(int bbx,int bby,float bbdx,float bbdy);
	virtual void bbDestroy();
	virtual void bbExplode();
	virtual void bbOnUpdate();
	virtual void bbOnRender();
	void mark();
};
class bb_powerup_PowerUp : public bb_gameobject_GameObject{
	public:
	bool bbdestroyed;
	float bbdx;
	float bbdy;
	float bbframe;
	bb_powerup_PowerUp();
	static bb_graphics_Image* bbimg;
	static void bbInit();
	virtual bool bbIsDestroyed();
	bb_powerup_PowerUp* bbnew();
	static bb_powerup_PowerUp* bbCreate(float bbx,float bby,float bbdx,float bbdy);
	virtual void bbOnRender();
	virtual void bbDestroy();
	virtual void bbOnUpdate();
	void mark();
};
extern int bb_random_Seed;
int bb_app_SetUpdateRate(int bbhertz);
int bb_graphics_SetFont(bb_graphics_Image* bbfont,int bbfirstChar);
int bb_graphics_PushMatrix();
int bb_graphics_Transform(float bbix,float bbiy,float bbjx,float bbjy,float bbtx,float bbty);
Array<float > bb_graphics_Transform2(Array<float > bbcoords);
int bb_graphics_Translate(float bbx,float bby);
int bb_graphics_ValidateMatrix();
int bb_graphics_PopMatrix();
int bb_graphics_DrawImage(bb_graphics_Image* bbimage,float bbx,float bby,int bbframe);
int bb_graphics_Rotate(float bbangle);
int bb_graphics_Scale(float bbx,float bby);
int bb_graphics_DrawImage2(bb_graphics_Image* bbimage,float bbx,float bby,float bbrotation,float bbscaleX,float bbscaleY,int bbframe);
int bb_graphics_DrawText(String bbtext,float bbx,float bby,float bbxalign,float bbyalign);
float bb_graphics_GetAlpha();
Array<float > bb_graphics_GetColor();
int bb_graphics_DrawRect(float bbx,float bby,float bbw,float bbh);
int bb_app_Millisecs();
int bb_audio_PlaySound(bb_audio_Sound* bbsound,int bbchannel,int bbflags);
int bb_input_MouseHit(int bbbutton);
int bb_graphics_DrawImageRect(bb_graphics_Image* bbimage,float bbx,float bby,int bbsrcX,int bbsrcY,int bbsrcWidth,int bbsrcHeight,int bbframe);
int bb_graphics_DrawImageRect2(bb_graphics_Image* bbimage,float bbx,float bby,int bbsrcX,int bbsrcY,int bbsrcWidth,int bbsrcHeight,float bbrotation,float bbscaleX,float bbscaleY,int bbframe);
float bb_random_Rnd();
float bb_random_Rnd2(float bblow,float bbhigh);
float bb_random_Rnd3(float bbrange);
class bb_collision_Collision : public Object{
	public:
	bb_collision_Collision();
	static bool bbIntersectCircle(float bbx1,float bby1,float bbr1,float bbx2,float bby2,float bbr2);
	static bool bbIntersectRect(float bbx1,float bby1,float bbw1,float bbh1,float bbx2,float bby2,float bbw2,float bbh2);
	void mark();
};
int bb_math_Abs(int bbx);
float bb_math_Abs2(float bbx);
int bb_audio_StopChannel(int bbchannel);
float bb_input_MouseX();
float bb_input_MouseY();
int bb_graphics_DrawLine(float bbx1,float bby1,float bbx2,float bby2);
int bb_input_MouseDown(int bbbutton);
bb_app_App::bb_app_App(){
}
bb_app_App* bb_app_App::bbnew(){
	bb_app_device=(new bb_app_AppDevice())->bbnew(this);
	return this;
}
int bb_app_App::bbOnCreate(){
	return 0;
}
int bb_app_App::bbOnUpdate(){
	return 0;
}
int bb_app_App::bbOnSuspend(){
	return 0;
}
int bb_app_App::bbOnResume(){
	return 0;
}
int bb_app_App::bbOnRender(){
	return 0;
}
int bb_app_App::bbOnLoading(){
	return 0;
}
void bb_app_App::mark(){
	Object::mark();
}
bb_application_Application::bb_application_Application(){
	bbfaders=0;
	bbscenes=0;
	bbcurrentScene=0;
	bbstate=0;
	bbloading=false;
	bbnextScene=0;
}
bb_application_Application* bb_application_Application::bbapplication;
bb_application_Application* bb_application_Application::bbnew(){
	bb_app_App::bbnew();
	return this;
}
bb_application_Application* bb_application_Application::bbGetInstance(){
	if(!((bbapplication)!=0)){
		bbapplication=(new bb_application_Application())->bbnew();
	}
	return bbapplication;
}
void bb_application_Application::bbInit(){
	bbfaders=(new bb_list_List())->bbnew();
	bbscenes=(new bb_map_StringMap())->bbnew();
}
void bb_application_Application::bbAddFader(bb_fader_Fader* bbf){
	bbfaders->bbAddLast(bbf);
}
void bb_application_Application::bbAddScene(String bbsceneName,bb_scene_Scene* bbscene){
	if(bbscenes->bbIsEmpty()){
		bbcurrentScene=bbscene;
	}
	bbscene->bbname=bbsceneName;
	bbscenes->bbSet(((new bb_boxes_StringObject())->bbnew3(bbsceneName)),bbscene);
}
void bb_application_Application::bbRun(){
	if(!((bbcurrentScene)!=0)){
		Error(T("No scenes found!"));
	}
	bbcurrentScene->bbOnEnter();
	bb_list_Enumerator* bb=bbfaders->bbObjectEnumerator();
	while(bb->bbHasNext()){
		bb_fader_Fader* bbf=dynamic_cast<bb_fader_Fader*>((Object*)(bb->bbNextObject()));
		bbf->bbFadeIn();
	}
}
int bb_application_Application::bbOnCreate(){
	bb_app_SetUpdateRate(60);
	return 0;
}
void bb_application_Application::bbSetState(int bbst){
	bbstate=bbst;
}
int bb_application_Application::bbOnUpdate(){
	bbcurrentScene->bbOnUpdate();
	if(bbstate!=1){
		bool bbchangeState=true;
		bb_list_Enumerator* bb=bbfaders->bbObjectEnumerator();
		while(bb->bbHasNext()){
			bb_fader_Fader* bbf=dynamic_cast<bb_fader_Fader*>((Object*)(bb->bbNextObject()));
			if(bbf->bbIsFading()){
				bbchangeState=false;
			}
			bbf->bbOnUpdate();
		}
		if(bbchangeState){
			if(bbstate==0 && bbloading==false){
				bbSetState(1);
			}else{
				if(bbstate==2){
					bbSetState(0);
					bbcurrentScene->bbOnLeave();
					bbcurrentScene=bbnextScene;
					bbcurrentScene->bbOnEnter();
					bb_list_Enumerator* bb2=bbfaders->bbObjectEnumerator();
					while(bb2->bbHasNext()){
						bb_fader_Fader* bbf2=dynamic_cast<bb_fader_Fader*>((Object*)(bb2->bbNextObject()));
						bbf2->bbFadeIn();
					}
				}
			}
		}
	}
	return 0;
}
int bb_application_Application::bbOnRender(){
	bbloading=false;
	if((bbcurrentScene)!=0){
		bbcurrentScene->bbOnRender();
	}
	if(bbstate!=1){
		bb_list_Enumerator* bb=bbfaders->bbObjectEnumerator();
		while(bb->bbHasNext()){
			bb_fader_Fader* bbf=dynamic_cast<bb_fader_Fader*>((Object*)(bb->bbNextObject()));
			bbf->bbOnRender();
		}
	}
	return 0;
}
int bb_application_Application::bbOnLoading(){
	bbloading=true;
	String bbt=T("Loading...Please stand by");
	bb_graphics_SetColor(0.000000f,255.000000f,255.000000f);
	bb_graphics_SetAlpha(0.5f);
	bb_graphics_DrawText(bbt,float(bb_graphics_DeviceWidth()/2),float(bb_graphics_DeviceHeight()-40),.5f,.5f);
	return 0;
}
int bb_application_Application::bbOnResume(){
	return 0;
}
int bb_application_Application::bbOnSuspend(){
	return 0;
}
void bb_application_Application::bbSetNextScene(String bbsceneName){
	if((bbnextScene)!=0){
		if(bbnextScene->bbname==bbsceneName){
			return;
		}
	}
	bbSetState(2);
	bbnextScene=dynamic_cast<bb_scene_Scene*>((Object*)(bbscenes->bbGet((((new bb_boxes_StringObject())->bbnew3(bbsceneName))))));
	bb_list_Enumerator* bb=bbfaders->bbObjectEnumerator();
	while(bb->bbHasNext()){
		bb_fader_Fader* bbf=dynamic_cast<bb_fader_Fader*>((Object*)(bb->bbNextObject()));
		bbf->bbFadeOut();
	}
}
void bb_application_Application::mark(){
	bb_app_App::mark();
	gc_mark(bbfaders);
	gc_mark(bbscenes);
	gc_mark(bbcurrentScene);
	gc_mark(bbnextScene);
}
bb_app_AppDevice::bb_app_AppDevice(){
	bbapp=0;
}
bb_app_AppDevice* bb_app_AppDevice::bbnew(bb_app_App* bbapp){
	this->bbapp=bbapp;
	bb_graphics_SetGraphicsContext((new bb_graphics_GraphicsContext())->bbnew(GraphicsDevice()));
	bb_input_SetInputDevice(InputDevice());
	bb_audio_SetAudioDevice(AudioDevice());
	return this;
}
bb_app_AppDevice* bb_app_AppDevice::bbnew2(){
	return this;
}
int bb_app_AppDevice::OnCreate(){
	return bbapp->bbOnCreate();
}
int bb_app_AppDevice::OnUpdate(){
	return bbapp->bbOnUpdate();
}
int bb_app_AppDevice::OnSuspend(){
	return bbapp->bbOnSuspend();
}
int bb_app_AppDevice::OnResume(){
	return bbapp->bbOnResume();
}
int bb_app_AppDevice::OnRender(){
	bb_graphics_BeginRender();
	int bbr=bbapp->bbOnRender();
	bb_graphics_EndRender();
	return bbr;
}
int bb_app_AppDevice::OnLoading(){
	bb_graphics_BeginRender();
	int bbr=bbapp->bbOnLoading();
	bb_graphics_EndRender();
	return bbr;
}
void bb_app_AppDevice::mark(){
	gxtkApp::mark();
	gc_mark(bbapp);
}
bb_graphics_GraphicsContext::bb_graphics_GraphicsContext(){
	bbdevice=0;
	bbmatrixSp=0;
	bbix=1.000000f;
	bbiy=0;
	bbjx=0;
	bbjy=1.000000f;
	bbtx=0;
	bbty=0;
	bbtformed=0;
	bbmatDirty=0;
	bbcolor_r=0;
	bbcolor_g=0;
	bbcolor_b=0;
	bbalpha=0;
	bbblend=0;
	bbscissor_x=0;
	bbscissor_y=0;
	bbscissor_width=0;
	bbscissor_height=0;
	bbfont=0;
	bbdefaultFont=0;
	bbfirstChar=0;
	bbmatrixStack=Array<float >(192);
}
bb_graphics_GraphicsContext* bb_graphics_GraphicsContext::bbnew(gxtkGraphics* bbdevice){
	this->bbdevice=bbdevice;
	return this;
}
bb_graphics_GraphicsContext* bb_graphics_GraphicsContext::bbnew2(){
	return this;
}
void bb_graphics_GraphicsContext::mark(){
	Object::mark();
	gc_mark(bbdevice);
	gc_mark(bbfont);
	gc_mark(bbdefaultFont);
	gc_mark(bbmatrixStack);
}
bb_graphics_GraphicsContext* bb_graphics_context;
int bb_graphics_SetGraphicsContext(bb_graphics_GraphicsContext* bbgc){
	bb_graphics_context=bbgc;
	return 0;
}
gxtkInput* bb_input_device;
int bb_input_SetInputDevice(gxtkInput* bbdev){
	bb_input_device=bbdev;
	return 0;
}
gxtkAudio* bb_audio_device;
int bb_audio_SetAudioDevice(gxtkAudio* bbdev){
	bb_audio_device=bbdev;
	return 0;
}
bb_app_AppDevice* bb_app_device;
bb_fader_Fader::bb_fader_Fader(){
	bbspeed=0;
	bbfadeState=0;
	bbposition=0;
}
bb_fader_Fader* bb_fader_Fader::bbnew(){
	bbspeed=0.05f;
	bbfadeState=2;
	bbposition=1.000000f;
	return this;
}
void bb_fader_Fader::bbFadeIn(){
	bbfadeState=2;
}
bool bb_fader_Fader::bbIsFading(){
	return bbfadeState!=1;
}
void bb_fader_Fader::bbOnUpdate(){
	int bb=bbfadeState;
	if(bb==0){
		bbposition+=bbspeed;
		if(bbposition>=1.000000f){
			bbposition=1.000000f;
			bbfadeState=1;
		}
	}else{
		if(bb==2){
			bbposition-=bbspeed;
			if(bbposition<=0.000000f){
				bbposition=0.000000f;
				bbfadeState=1;
			}
		}
	}
}
void bb_fader_Fader::bbOnRender(){
}
void bb_fader_Fader::bbFadeOut(){
	bbfadeState=0;
}
void bb_fader_Fader::mark(){
	Object::mark();
}
bb_list_List::bb_list_List(){
	bb_head=(new bb_list_Node())->bbnew();
}
bb_list_List* bb_list_List::bbnew(){
	return this;
}
bb_list_Node* bb_list_List::bbAddLast(Object* bbdata){
	return (new bb_list_Node())->bbnew2(bb_head,bb_head->bb_pred,bbdata);
}
bb_list_Enumerator* bb_list_List::bbObjectEnumerator(){
	return (new bb_list_Enumerator())->bbnew(this);
}
int bb_list_List::bbClear(){
	bb_head=(new bb_list_Node())->bbnew();
	return 0;
}
int bb_list_List::bbEquals(Object* bblhs,Object* bbrhs){
	return ((bblhs==bbrhs)?1:0);
}
int bb_list_List::bbRemoveEach(Object* bbvalue){
	bb_list_Node* bbnode=bb_head->bb_succ;
	while(bbnode!=bb_head){
		bbnode=bbnode->bb_succ;
		if(((bbEquals((bbnode->bb_pred->bb_data),(bbvalue)))!=0)){
			bbnode->bb_pred->bbRemove();
		}
	}
	return 0;
}
int bb_list_List::bbRemove(Object* bbvalue){
	bbRemoveEach(bbvalue);
	return 0;
}
void bb_list_List::mark(){
	Object::mark();
	gc_mark(bb_head);
}
bb_scene_Scene::bb_scene_Scene(){
	bbname=T("");
}
bb_scene_Scene* bb_scene_Scene::bbnew(){
	return this;
}
void bb_scene_Scene::bbOnEnter(){
}
void bb_scene_Scene::bbOnUpdate(){
}
void bb_scene_Scene::bbOnLeave(){
}
void bb_scene_Scene::bbOnRender(){
}
void bb_scene_Scene::mark(){
	Object::mark();
}
bb_boxes_StringObject::bb_boxes_StringObject(){
	bbvalue=T("");
}
bb_boxes_StringObject* bb_boxes_StringObject::bbnew(int bbvalue){
	this->bbvalue=String(bbvalue);
	return this;
}
bb_boxes_StringObject* bb_boxes_StringObject::bbnew2(float bbvalue){
	this->bbvalue=String(bbvalue);
	return this;
}
bb_boxes_StringObject* bb_boxes_StringObject::bbnew3(String bbvalue){
	this->bbvalue=bbvalue;
	return this;
}
bb_boxes_StringObject* bb_boxes_StringObject::bbnew4(){
	return this;
}
void bb_boxes_StringObject::mark(){
	Object::mark();
}
bb_map_Map::bb_map_Map(){
	bbroot=0;
}
bb_map_Map* bb_map_Map::bbnew(){
	return this;
}
int bb_map_Map::bbCompare(Object* bblhs,Object* bbrhs){
	return 0;
}
bb_map_Node* bb_map_Map::bbFindNode(Object* bbkey){
	bb_map_Node* bbnode=bbroot;
	while((bbnode)!=0){
		int bbcmp=bbCompare((bbkey),(bbnode->bbkey));
		if(bbcmp>0){
			bbnode=bbnode->bbright;
		}else{
			if(bbcmp<0){
				bbnode=bbnode->bbleft;
			}else{
				return bbnode;
			}
		}
	}
	return bbnode;
}
Object* bb_map_Map::bbGet(Object* bbkey){
	bb_map_Node* bbnode=bbFindNode(bbkey);
	if((bbnode)!=0){
		return bbnode->bbvalue;
	}
	return 0;
}
Object* bb_map_Map::bbValueForKey(Object* bbkey){
	return bbGet(bbkey);
}
int bb_map_Map::bbRotateLeft(bb_map_Node* bbnode){
	bb_map_Node* bbchild=bbnode->bbright;
	bbnode->bbright=bbchild->bbleft;
	if((bbchild->bbleft)!=0){
		bbchild->bbleft->bbparent=bbnode;
	}
	bbchild->bbparent=bbnode->bbparent;
	if((bbnode->bbparent)!=0){
		if(bbnode==bbnode->bbparent->bbleft){
			bbnode->bbparent->bbleft=bbchild;
		}else{
			bbnode->bbparent->bbright=bbchild;
		}
	}else{
		bbroot=bbchild;
	}
	bbchild->bbleft=bbnode;
	bbnode->bbparent=bbchild;
	return 0;
}
int bb_map_Map::bbRotateRight(bb_map_Node* bbnode){
	bb_map_Node* bbchild=bbnode->bbleft;
	bbnode->bbleft=bbchild->bbright;
	if((bbchild->bbright)!=0){
		bbchild->bbright->bbparent=bbnode;
	}
	bbchild->bbparent=bbnode->bbparent;
	if((bbnode->bbparent)!=0){
		if(bbnode==bbnode->bbparent->bbright){
			bbnode->bbparent->bbright=bbchild;
		}else{
			bbnode->bbparent->bbleft=bbchild;
		}
	}else{
		bbroot=bbchild;
	}
	bbchild->bbright=bbnode;
	bbnode->bbparent=bbchild;
	return 0;
}
int bb_map_Map::bbInsertFixup(bb_map_Node* bbnode){
	while(((bbnode->bbparent)!=0) && bbnode->bbparent->bbcolor==-1 && ((bbnode->bbparent->bbparent)!=0)){
		if(bbnode->bbparent==bbnode->bbparent->bbparent->bbleft){
			bb_map_Node* bbuncle=bbnode->bbparent->bbparent->bbright;
			if(((bbuncle)!=0) && bbuncle->bbcolor==-1){
				bbnode->bbparent->bbcolor=1;
				bbuncle->bbcolor=1;
				bbuncle->bbparent->bbcolor=-1;
				bbnode=bbuncle->bbparent;
			}else{
				if(bbnode==bbnode->bbparent->bbright){
					bbnode=bbnode->bbparent;
					bbRotateLeft(bbnode);
				}
				bbnode->bbparent->bbcolor=1;
				bbnode->bbparent->bbparent->bbcolor=-1;
				bbRotateRight(bbnode->bbparent->bbparent);
			}
		}else{
			bb_map_Node* bbuncle2=bbnode->bbparent->bbparent->bbleft;
			if(((bbuncle2)!=0) && bbuncle2->bbcolor==-1){
				bbnode->bbparent->bbcolor=1;
				bbuncle2->bbcolor=1;
				bbuncle2->bbparent->bbcolor=-1;
				bbnode=bbuncle2->bbparent;
			}else{
				if(bbnode==bbnode->bbparent->bbleft){
					bbnode=bbnode->bbparent;
					bbRotateRight(bbnode);
				}
				bbnode->bbparent->bbcolor=1;
				bbnode->bbparent->bbparent->bbcolor=-1;
				bbRotateLeft(bbnode->bbparent->bbparent);
			}
		}
	}
	bbroot->bbcolor=1;
	return 0;
}
int bb_map_Map::bbSet(Object* bbkey,Object* bbvalue){
	bb_map_Node* bbnode=bbroot;
	bb_map_Node* bbparent=0;
	int bbcmp=0;
	while((bbnode)!=0){
		bbparent=bbnode;
		bbcmp=bbCompare((bbkey),(bbnode->bbkey));
		if(bbcmp>0){
			bbnode=bbnode->bbright;
		}else{
			if(bbcmp<0){
				bbnode=bbnode->bbleft;
			}else{
				bbnode->bbvalue=bbvalue;
				return 0;
			}
		}
	}
	bbnode=(new bb_map_Node())->bbnew(bbkey,bbvalue,-1,bbparent);
	if(!((bbparent)!=0)){
		bbroot=bbnode;
		return 0;
	}
	if(bbcmp>0){
		bbparent->bbright=bbnode;
	}else{
		bbparent->bbleft=bbnode;
	}
	bbInsertFixup(bbnode);
	return 0;
}
int bb_map_Map::bbInsert(Object* bbkey,Object* bbvalue){
	return bbSet(bbkey,bbvalue);
}
bool bb_map_Map::bbIsEmpty(){
	return bbroot==0;
}
bool bb_map_Map::bbContains(Object* bbkey){
	return bbFindNode(bbkey)!=0;
}
void bb_map_Map::mark(){
	Object::mark();
	gc_mark(bbroot);
}
bb_map_StringMap::bb_map_StringMap(){
}
bb_map_StringMap* bb_map_StringMap::bbnew(){
	bb_map_Map::bbnew();
	return this;
}
int bb_map_StringMap::bbCompare(Object* bblhs,Object* bbrhs){
	String bbl=dynamic_cast<bb_boxes_StringObject*>((Object*)(bblhs))->bbvalue;
	String bbr=dynamic_cast<bb_boxes_StringObject*>((Object*)(bbrhs))->bbvalue;
	if(bbl<bbr){
		return -1;
	}
	return ((bbl>bbr)?1:0);
}
void bb_map_StringMap::mark(){
	bb_map_Map::mark();
}
bb_faderbrightness_FaderBrightness::bb_faderbrightness_FaderBrightness(){
}
bb_faderbrightness_FaderBrightness* bb_faderbrightness_FaderBrightness::bbnew(){
	bb_fader_Fader::bbnew();
	return this;
}
void bb_faderbrightness_FaderBrightness::bbOnRender(){
	float bbalpha=bb_graphics_GetAlpha();
	Array<float > bbc=bb_graphics_GetColor();
	bb_graphics_Scale(1.000000f,1.000000f);
	bb_graphics_SetAlpha(bbposition);
	bb_graphics_SetColor(0.000000f,0.000000f,0.000000f);
	bb_graphics_Rotate(0.000000f);
	bb_graphics_DrawRect(0.000000f,0.000000f,float(bb_graphics_DeviceWidth()),float(bb_graphics_DeviceHeight()));
	bb_graphics_SetColor(bbc[0],bbc[1],bbc[2]);
	bb_graphics_SetAlpha(bbalpha);
}
void bb_faderbrightness_FaderBrightness::mark(){
	bb_fader_Fader::mark();
}
bb_list_Node::bb_list_Node(){
	bb_succ=0;
	bb_pred=0;
	bb_data=0;
}
bb_list_Node* bb_list_Node::bbnew(){
	bb_succ=this;
	bb_pred=this;
	return this;
}
bb_list_Node* bb_list_Node::bbnew2(bb_list_Node* bbsucc,bb_list_Node* bbpred,Object* bbdata){
	bb_succ=bbsucc;
	bb_pred=bbpred;
	bb_succ->bb_pred=this;
	bb_pred->bb_succ=this;
	bb_data=bbdata;
	return this;
}
int bb_list_Node::bbRemove(){
	bb_succ->bb_pred=bb_pred;
	bb_pred->bb_succ=bb_succ;
	return 0;
}
void bb_list_Node::mark(){
	Object::mark();
	gc_mark(bb_succ);
	gc_mark(bb_pred);
	gc_mark(bb_data);
}
bb_soundmanager_SoundManager::bb_soundmanager_SoundManager(){
}
bb_audio_Sound* bb_soundmanager_SoundManager::bbbackgroundMusic;
Array<bb_audio_Sound* > bb_soundmanager_SoundManager::bbsfx;
void bb_soundmanager_SoundManager::bbLoadSounds(){
	bbbackgroundMusic=bb_audio_LoadSound(T("sfx/theme.wav"));
	bbsfx[0]=bb_audio_LoadSound(T("sfx/rocket.wav"));
	bbsfx[1]=bb_audio_LoadSound(T("sfx/xplode.wav"));
}
void bb_soundmanager_SoundManager::bbPlayMusic(){
	bb_audio_PlaySound(bbbackgroundMusic,0,1);
}
void bb_soundmanager_SoundManager::bbPlay(int bbsound){
	bb_audio_StopChannel(bbsound+1);
	bb_audio_PlaySound(bbsfx[bbsound],bbsound+1,0);
}
void bb_soundmanager_SoundManager::mark(){
	Object::mark();
}
bb_resource_Resource::bb_resource_Resource(){
	bbnode=0;
	bbrefs=1;
}
bb_resource_Resource* bb_resource_Resource::bbnew(){
	return this;
}
int bb_resource_Resource::bbRegister(String bbtype){
	bb_list_List* bblist=dynamic_cast<bb_list_List*>((Object*)(bb_resource_resources->bbValueForKey((((new bb_boxes_StringObject())->bbnew3(bbtype))))));
	if(!((bblist)!=0)){
		bblist=(new bb_list_List())->bbnew();
		bb_resource_resources->bbInsert(((new bb_boxes_StringObject())->bbnew3(bbtype)),bblist);
	}
	bbnode=bblist->bbAddLast(this);
	return 0;
}
int bb_resource_Resource::bbRetain(){
	bbrefs+=1;
	return 0;
}
void bb_resource_Resource::mark(){
	Object::mark();
	gc_mark(bbnode);
}
bb_audio_Sound::bb_audio_Sound(){
	bbsample=0;
}
bb_audio_Sound* bb_audio_Sound::bbnew(gxtkSample* bbsample){
	bb_resource_Resource::bbnew();
	this->bbsample=bbsample;
	if((bbsample)!=0){
		bbRegister(T("mojo.audio.Sound"));
	}
	return this;
}
bb_audio_Sound* bb_audio_Sound::bbnew2(){
	bb_resource_Resource::bbnew();
	return this;
}
void bb_audio_Sound::mark(){
	bb_resource_Resource::mark();
	gc_mark(bbsample);
}
bb_map_StringMap* bb_resource_resources;
bb_map_Node::bb_map_Node(){
	bbkey=0;
	bbright=0;
	bbleft=0;
	bbvalue=0;
	bbcolor=0;
	bbparent=0;
}
bb_map_Node* bb_map_Node::bbnew(Object* bbkey,Object* bbvalue,int bbcolor,bb_map_Node* bbparent){
	this->bbkey=bbkey;
	this->bbvalue=bbvalue;
	this->bbcolor=bbcolor;
	this->bbparent=bbparent;
	return this;
}
bb_map_Node* bb_map_Node::bbnew2(){
	return this;
}
void bb_map_Node::mark(){
	Object::mark();
	gc_mark(bbkey);
	gc_mark(bbright);
	gc_mark(bbleft);
	gc_mark(bbvalue);
	gc_mark(bbparent);
}
bb_audio_Sound* bb_audio_LoadSound(String bbpath){
	gxtkSample* bbsample=bb_audio_device->LoadSample(bbpath);
	return (new bb_audio_Sound())->bbnew(bbsample);
}
bb_gamefonts_GameFonts::bb_gamefonts_GameFonts(){
}
bb_angelfont_AngelFont* bb_gamefonts_GameFonts::bbnormal;
void bb_gamefonts_GameFonts::bbLoad(){
	bbnormal=(new bb_angelfont_AngelFont())->bbnew2();
	bbnormal->bbLoadFont(T("fonts/angel_verdana"));
}
void bb_gamefonts_GameFonts::mark(){
	Object::mark();
}
bb_angelfont_AngelFont::bb_angelfont_AngelFont(){
	bbiniText=T("");
	bbkernPairs=(new bb_map_StringMap())->bbnew();
	bbchars=Array<bb_char_Char* >(256);
	bbheight=0;
	bbheightOffset=9999;
	bbimage=0;
	bbname=T("");
	bbxOffset=0;
	bbuseKerning=true;
}
String bb_angelfont_AngelFont::bberror;
bb_angelfont_AngelFont* bb_angelfont_AngelFont::bbcurrent;
void bb_angelfont_AngelFont::bbLoadFont(String bburl){
	bberror=T("");
	bbcurrent=this;
	bbiniText=bb_app_LoadString(bburl+T(".txt"));
	Array<String > bblines=bbiniText.Split(String((Char)(10),1));
	Array<String > bb=bblines;
	int bb2=0;
	while(bb2<bb.Length()){
		String bbline=bb[bb2];
		bb2=bb2+1;
		if(bbline.StartsWith(T("id,")) || bbline==T("")){
			continue;
		}
		if(bbline.StartsWith(T("first,"))){
			continue;
		}
		Array<String > bbdata=bbline.Split(T(","));
		bberror+=String(bbdata.Length())+T(",");
		if(bbdata.Length()>0){
			if(bbdata.Length()==3){
				bbkernPairs->bbInsert(((new bb_boxes_StringObject())->bbnew3(String((Char)((bbdata[0]).ToInt()),1)+T("_")+String((Char)((bbdata[1]).ToInt()),1))),(new bb_kernpair_KernPair())->bbnew((bbdata[0]).ToInt(),(bbdata[1]).ToInt(),(bbdata[2]).ToInt()));
			}else{
				if(bbdata.Length()>=8){
					bbchars[(bbdata[0]).ToInt()]=(new bb_char_Char())->bbnew((bbdata[1]).ToInt(),(bbdata[2]).ToInt(),(bbdata[3]).ToInt(),(bbdata[4]).ToInt(),(bbdata[5]).ToInt(),(bbdata[6]).ToInt(),(bbdata[7]).ToInt());
					bb_char_Char* bbch=bbchars[(bbdata[0]).ToInt()];
					if(bbch->bbheight>this->bbheight){
						this->bbheight=bbch->bbheight;
					}
					if(bbch->bbyOffset<this->bbheightOffset){
						this->bbheightOffset=bbch->bbyOffset;
					}
				}
			}
		}
	}
	bbimage=bb_graphics_LoadImage(bburl+T(".png"),1,bb_graphics_Image::bbDefaultFlags);
}
bb_map_StringMap* bb_angelfont_AngelFont::bb_list;
bb_angelfont_AngelFont* bb_angelfont_AngelFont::bbnew(String bburl){
	if(bburl!=T("")){
		this->bbLoadFont(bburl);
		this->bbname=bburl;
		bb_list->bbInsert(((new bb_boxes_StringObject())->bbnew3(bburl)),this);
	}
	return this;
}
bb_angelfont_AngelFont* bb_angelfont_AngelFont::bbnew2(){
	return this;
}
void bb_angelfont_AngelFont::bbDrawText(String bbtxt,int bbx,int bby){
	String bbprevChar=T("");
	bbxOffset=0;
	for(int bbi=0;bbi<bbtxt.Length();bbi=bbi+1){
		int bbasc=(int)bbtxt[bbi];
		bb_char_Char* bbac=bbchars[bbasc];
		String bbthisChar=String((Char)(bbasc),1);
		if(bbac!=0){
			if(bbuseKerning){
				String bbkey=bbprevChar+T("_")+bbthisChar;
				if(bbkernPairs->bbContains((((new bb_boxes_StringObject())->bbnew3(bbkey))))){
					bbxOffset+=dynamic_cast<bb_kernpair_KernPair*>((Object*)(bbkernPairs->bbGet((((new bb_boxes_StringObject())->bbnew3(bbkey))))))->bbamount;
				}
			}
			bbac->bbDraw(bbimage,bbx+bbxOffset,bby);
			bbxOffset+=bbac->bbxAdvance;
			bbprevChar=bbthisChar;
		}
	}
}
int bb_angelfont_AngelFont::bbTextWidth(String bbtxt){
	String bbprevChar=T("");
	int bbwidth=0;
	for(int bbi=0;bbi<bbtxt.Length();bbi=bbi+1){
		int bbasc=(int)bbtxt[bbi];
		bb_char_Char* bbac=bbchars[bbasc];
		String bbthisChar=String((Char)(bbasc),1);
		if(bbac!=0){
			if(bbuseKerning){
				String bbkey=bbprevChar+T("_")+bbthisChar;
				if(bbkernPairs->bbContains((((new bb_boxes_StringObject())->bbnew3(bbkey))))){
					bbwidth+=dynamic_cast<bb_kernpair_KernPair*>((Object*)(bbkernPairs->bbGet((((new bb_boxes_StringObject())->bbnew3(bbkey))))))->bbamount;
				}
			}
			bbwidth+=bbac->bbxAdvance;
			bbprevChar=bbthisChar;
		}
	}
	return bbwidth;
}
void bb_angelfont_AngelFont::bbDrawText2(String bbtxt,int bbx,int bby,int bbalign){
	bbxOffset=0;
	int bb=bbalign;
	if(bb==1){
		bbDrawText(bbtxt,bbx-bbTextWidth(bbtxt)/2,bby);
	}else{
		if(bb==2){
			bbDrawText(bbtxt,bbx-bbTextWidth(bbtxt),bby);
		}else{
			if(bb==0){
				bbDrawText(bbtxt,bbx,bby);
			}
		}
	}
}
void bb_angelfont_AngelFont::mark(){
	Object::mark();
	gc_mark(bbkernPairs);
	gc_mark(bbchars);
	gc_mark(bbimage);
}
String bb_app_LoadString(String bbpath){
	return bb_app_device->LoadString(bbpath);
}
bb_kernpair_KernPair::bb_kernpair_KernPair(){
	bbfirst=T("");
	bbsecond=T("");
	bbamount=0;
}
bb_kernpair_KernPair* bb_kernpair_KernPair::bbnew(int bbfirst,int bbsecond,int bbamount){
	this->bbfirst=String(bbfirst);
	this->bbsecond=String(bbsecond);
	this->bbamount=bbamount;
	return this;
}
bb_kernpair_KernPair* bb_kernpair_KernPair::bbnew2(){
	return this;
}
void bb_kernpair_KernPair::mark(){
	Object::mark();
}
bb_char_Char::bb_char_Char(){
	bbx=0;
	bby=0;
	bbwidth=0;
	bbheight=0;
	bbxOffset=0;
	bbyOffset=0;
	bbxAdvance=0;
}
bb_char_Char* bb_char_Char::bbnew(int bbx,int bby,int bbw,int bbh,int bbxoff,int bbyoff,int bbxadv){
	this->bbx=bbx;
	this->bby=bby;
	this->bbwidth=bbw;
	this->bbheight=bbh;
	this->bbxOffset=bbxoff;
	this->bbyOffset=bbyoff;
	this->bbxAdvance=bbxadv;
	return this;
}
bb_char_Char* bb_char_Char::bbnew2(){
	return this;
}
int bb_char_Char::bbDraw(bb_graphics_Image* bbfontImage,int bblinex,int bbliney){
	bb_graphics_DrawImageRect(bbfontImage,float(bblinex+bbxOffset),float(bbliney+bbyOffset),bbx,bby,bbwidth,bbheight,0);
	return 0;
}
void bb_char_Char::mark(){
	Object::mark();
}
bb_graphics_Image::bb_graphics_Image(){
	bbsurface=0;
	bbwidth=0;
	bbheight=0;
	bbflags=0;
	bbframes=Array<bb_graphics_Frame* >();
	bbtx=0;
	bbty=0;
	bbsource=0;
}
int bb_graphics_Image::bbDefaultFlags;
int bb_graphics_Image::bbSetHandle(float bbtx,float bbty){
	this->bbtx=bbtx;
	this->bbty=bbty;
	this->bbflags=this->bbflags&-2;
	return 0;
}
bb_graphics_Image* bb_graphics_Image::bbnew(String bbpath,int bbnframes,int bbiflags){
	bb_resource_Resource::bbnew();
	bbsurface=bb_graphics_context->bbdevice->LoadSurface(bbpath);
	if(!((bbsurface)!=0)){
		Error(T("Failed to load image ")+bbpath);
	}
	bbRegister(T("mojo.graphics.Image"));
	bbwidth=bbsurface->Width()/bbnframes;
	bbheight=bbsurface->Height();
	bbflags=bbiflags;
	bbframes=Array<bb_graphics_Frame* >(bbnframes);
	for(int bbi=0;bbi<bbnframes;bbi=bbi+1){
		bbframes[bbi]=(new bb_graphics_Frame())->bbnew(bbi*bbwidth,0);
	}
	if(bbnframes==1){
		bbflags|=65536;
	}
	if((bbflags&1)!=0){
		bbSetHandle(float(bbwidth)/2.0f,float(bbheight)/2.0f);
	}
	return this;
}
bb_graphics_Image* bb_graphics_Image::bbnew2(int bbx,int bby,int bbiwidth,int bbiheight,int bbnframes,int bbiflags,bb_graphics_Image* bbsource){
	bb_resource_Resource::bbnew();
	bbRegister(T("mojo.graphics.Image"));
	bbsource->bbRetain();
	this->bbsource=bbsource;
	bbsurface=bbsource->bbsurface;
	bbwidth=bbiwidth;
	bbheight=bbiheight;
	bbflags=bbiflags;
	bbframes=Array<bb_graphics_Frame* >(bbnframes);
	int bbix=bbx+bbsource->bbframes[0]->bbx;
	int bbiy=bby+bbsource->bbframes[0]->bby;
	for(int bbi=0;bbi<bbnframes;bbi=bbi+1){
		if(bbix+bbwidth>bbsource->bbwidth){
			bbix=bbsource->bbframes[0]->bbx;
			bbiy+=bbheight;
		}
		bbframes[bbi]=(new bb_graphics_Frame())->bbnew(bbix,bbiy);
		bbix+=bbwidth;
	}
	if(bbnframes==1 && bbx==0 && bby==0 && bbwidth==bbsurface->Width() && bbheight==bbsurface->Height()){
		bbflags|=65536;
	}
	if((bbflags&1)!=0){
		bbSetHandle(float(bbwidth)/2.0f,float(bbheight)/2.0f);
	}
	return this;
}
bb_graphics_Image* bb_graphics_Image::bbnew3(){
	bb_resource_Resource::bbnew();
	return this;
}
bb_graphics_Image* bb_graphics_Image::bbGrabImage(int bbx,int bby,int bbwidth,int bbheight,int bbframes,int bbflags){
	if(this->bbframes.Length()!=1){
		return 0;
	}
	return (new bb_graphics_Image())->bbnew2(bbx,bby,bbwidth,bbheight,bbframes,bbflags,this);
}
int bb_graphics_Image::bbWidth(){
	return bbwidth;
}
int bb_graphics_Image::bbLoaded(){
	return bbsurface->Loaded();
}
int bb_graphics_Image::bbHeight(){
	return bbheight;
}
int bb_graphics_Image::bbFrames(){
	return bbframes.Length();
}
void bb_graphics_Image::mark(){
	bb_resource_Resource::mark();
	gc_mark(bbsurface);
	gc_mark(bbframes);
	gc_mark(bbsource);
}
bb_graphics_Frame::bb_graphics_Frame(){
	bbx=0;
	bby=0;
}
bb_graphics_Frame* bb_graphics_Frame::bbnew(int bbx,int bby){
	this->bbx=bbx;
	this->bby=bby;
	return this;
}
bb_graphics_Frame* bb_graphics_Frame::bbnew2(){
	return this;
}
void bb_graphics_Frame::mark(){
	Object::mark();
}
bb_graphics_Image* bb_graphics_LoadImage(String bbpath,int bbframeCount,int bbflags){
	return (new bb_graphics_Image())->bbnew(bbpath,bbframeCount,bbflags);
}
bb_graphics_Image* bb_graphics_LoadImage2(String bbpath,int bbframeWidth,int bbframeHeight,int bbframeCount,int bbflags){
	bb_graphics_Image* bbatlas=(new bb_graphics_Image())->bbnew(bbpath,1,0);
	if((bbatlas)!=0){
		return bbatlas->bbGrabImage(0,0,bbframeWidth,bbframeHeight,bbframeCount,bbflags);
	}
	return 0;
}
bb_scenelogo_SceneLogo::bb_scenelogo_SceneLogo(){
	bblogo=0;
	bbtimer=0;
}
bb_scenelogo_SceneLogo* bb_scenelogo_SceneLogo::bbnew(){
	bb_scene_Scene::bbnew();
	return this;
}
void bb_scenelogo_SceneLogo::bbOnEnter(){
	bblogo=bb_graphics_LoadImage(T("gfx/logo.png"),1,bb_graphics_Image::bbDefaultFlags);
}
void bb_scenelogo_SceneLogo::bbOnUpdate(){
	if(bbtimer==0){
		bbtimer=bb_app_Millisecs()+3000;
		bb_soundmanager_SoundManager::bbPlayMusic();
	}
	if(bbtimer<bb_app_Millisecs()){
		bb_application_Application::bbGetInstance()->bbSetNextScene(T("title"));
	}
}
void bb_scenelogo_SceneLogo::bbOnRender(){
	bb_graphics_DrawImage(bblogo,float((bb_graphics_DeviceWidth()/2-bblogo->bbWidth()/2)),float((bb_graphics_DeviceHeight()/2-bblogo->bbHeight()/2)),0);
}
void bb_scenelogo_SceneLogo::bbOnLeave(){
	bblogo=0;
}
void bb_scenelogo_SceneLogo::mark(){
	bb_scene_Scene::mark();
	gc_mark(bblogo);
}
bb_scenetitle_SceneTitle::bb_scenetitle_SceneTitle(){
	bbtitlepic=0;
}
bb_scenetitle_SceneTitle* bb_scenetitle_SceneTitle::bbnew(){
	bb_scene_Scene::bbnew();
	return this;
}
void bb_scenetitle_SceneTitle::bbOnEnter(){
	bbtitlepic=bb_graphics_LoadImage(T("gfx/title.png"),1,bb_graphics_Image::bbDefaultFlags);
}
void bb_scenetitle_SceneTitle::bbOnUpdate(){
	if((bb_input_MouseHit(0))!=0){
		bb_application_Application::bbGetInstance()->bbSetNextScene(T("game"));
	}
}
void bb_scenetitle_SceneTitle::bbOnRender(){
	bb_graphics_DrawImage(bbtitlepic,0.000000f,0.000000f,0);
	if(bb_app_Millisecs()/1000 % 2==0){
		bb_gamefonts_GameFonts::bbnormal->bbDrawText(T("Click to start a new game"),66,227);
	}
	bb_graphics_PushMatrix();
	bb_graphics_Scale(0.6f,0.6f);
	bb_gamefonts_GameFonts::bbnormal->bbDrawText(T("Code by @joemanaco      GFX by @Emme73"),100,450);
	bb_graphics_PopMatrix();
}
void bb_scenetitle_SceneTitle::bbOnLeave(){
}
void bb_scenetitle_SceneTitle::mark(){
	bb_scene_Scene::mark();
	gc_mark(bbtitlepic);
}
bb_scenegame_SceneGame::bb_scenegame_SceneGame(){
	bblevel=0;
	bbscore=0;
	bbstate=0;
	bbbackground=Array<bb_graphics_Image* >(2);
	bblaunchDelay=0;
	bbtimer=0;
	bblaunchAmount=0;
	bblaunchDx=0;
	bblaunchDy=0;
	bblaunchCount=0;
}
bb_scenegame_SceneGame* bb_scenegame_SceneGame::bbnew(){
	bb_scene_Scene::bbnew();
	return this;
}
void bb_scenegame_SceneGame::bbInitGame(){
	bblevel=0;
	bbscore=0;
	bbstate=1;
	bbbackground[0]=bb_graphics_LoadImage(T("gfx/bg01.png"),1,bb_graphics_Image::bbDefaultFlags);
	bbbackground[1]=bb_graphics_LoadImage(T("gfx/bg02.png"),1,bb_graphics_Image::bbDefaultFlags);
	bb_base_Base::bbCreate();
	bb_station_Station::bbCreate(11,255,0);
	bb_station_Station::bbCreate(235,248,1);
	bb_station_Station::bbCreate(331,248,1);
	bb_station_Station::bbCreate(416,248,1);
}
void bb_scenegame_SceneGame::bbInitLevel(int bbl){
	bblaunchDelay=60*(6-bbl/2);
	bbtimer=bblaunchDelay;
	bblaunchAmount=2+bbl/3;
	bblaunchDx=0.2f+float(bbl)*0.1f;
	bblaunchDy=bblaunchDx*1.5f+float(bbl)*0.1f;
	bb_random_Seed=bblevel;
	bblaunchCount=3+bbl*2;
}
void bb_scenegame_SceneGame::bbOnEnter(){
	bb_gameobject_GameObject::bbInit();
	bb_base_Base::bbInit();
	bb_station_Station::bbInit();
	bb_bullet_Bullet::bbInit();
	bb_explosion_Explosion::bbInit();
	bb_asteroid_Asteroid::bbInit();
	bb_powerup_PowerUp::bbInit();
	bbInitGame();
	bbInitLevel(bblevel);
}
void bb_scenegame_SceneGame::bbClearLevel(){
	bb_list_Enumerator* bb=bb_gameobject_GameObject::bblist->bbObjectEnumerator();
	while(bb->bbHasNext()){
		bb_gameobject_GameObject* bbgo=dynamic_cast<bb_gameobject_GameObject*>((Object*)(bb->bbNextObject()));
		bb_asteroid_Asteroid* bba=dynamic_cast<bb_asteroid_Asteroid*>((bb_gameobject_GameObject*)(bbgo));
		bb_bullet_Bullet* bbb=dynamic_cast<bb_bullet_Bullet*>((bb_gameobject_GameObject*)(bbgo));
		bb_explosion_Explosion* bbe=dynamic_cast<bb_explosion_Explosion*>((bb_gameobject_GameObject*)(bbgo));
		if((bba)!=0){
			bba->bbDestroy();
		}
		if((bbb)!=0){
			bbb->bbDestroy();
		}
		if((bbe)!=0){
			bbe->bbDestroy();
		}
	}
}
void bb_scenegame_SceneGame::bbCheckCollision(){
	int bbstationsLeft=0;
	bool bbexplode=false;
	bb_list_Enumerator* bb=bb_gameobject_GameObject::bblist->bbObjectEnumerator();
	while(bb->bbHasNext()){
		bb_gameobject_GameObject* bba=dynamic_cast<bb_gameobject_GameObject*>((Object*)(bb->bbNextObject()));
		if(((dynamic_cast<bb_station_Station*>((bb_gameobject_GameObject*)(bba)))!=0) && !dynamic_cast<bb_station_Station*>((bb_gameobject_GameObject*)(bba))->bbIsDestroyed()){
			bbstationsLeft+=1;
		}
		if(((dynamic_cast<bb_asteroid_Asteroid*>((bb_gameobject_GameObject*)(bba)))!=0)){
			bb_asteroid_Asteroid* bbasteroid=dynamic_cast<bb_asteroid_Asteroid*>((bb_gameobject_GameObject*)(bba));
			if(!bba->bbIsDestroyed()){
				bb_list_Enumerator* bb2=bb_gameobject_GameObject::bblist->bbObjectEnumerator();
				while(bb2->bbHasNext()){
					bb_gameobject_GameObject* bbobj=dynamic_cast<bb_gameobject_GameObject*>((Object*)(bb2->bbNextObject()));
					if(!bbobj->bbIsDestroyed()){
						if(((dynamic_cast<bb_bullet_Bullet*>((bb_gameobject_GameObject*)(bbobj)))!=0)){
							if(bb_collision_Collision::bbIntersectCircle(bba->bbx,bba->bby,bb_asteroid_Asteroid::bbcollisionRadius,bbobj->bbx,bbobj->bby,float(bb_bullet_Bullet::bbcollisionRadius))){
								bbasteroid->bbExplode();
								bbobj->bbDestroy();
								bbscore+=50;
								bbexplode=true;
							}
						}else{
							if(((dynamic_cast<bb_explosion_Explosion*>((bb_gameobject_GameObject*)(bbobj)))!=0)){
								if(bb_collision_Collision::bbIntersectCircle(bba->bbx,bba->bby,bb_asteroid_Asteroid::bbcollisionRadius,bbobj->bbx,bbobj->bby,bb_explosion_Explosion::bbcollisionRadius)){
									if(bb_random_Rnd2(0.000000f,100.000000f)>95.000000f){
										bb_powerup_PowerUp::bbCreate(bba->bbx,bba->bby,(bbobj->bbx-bba->bbx)/20.000000f,-bb_math_Abs2((bbobj->bby-bba->bby)/10.000000f));
									}
									bbasteroid->bbExplode();
									bbscore+=75;
									bbexplode=true;
								}
							}else{
								if(((dynamic_cast<bb_station_Station*>((bb_gameobject_GameObject*)(bbobj)))!=0)){
									int bbx=int((bbobj->bbx+(float((bb_station_Station::bbimg[dynamic_cast<bb_station_Station*>((bb_gameobject_GameObject*)(bbobj))->bbtype]->bbWidth()))-bb_station_Station::bbcollisionRadius)/2.000000f));
									int bby=int((bbobj->bby+(float((bb_station_Station::bbimg[dynamic_cast<bb_station_Station*>((bb_gameobject_GameObject*)(bbobj))->bbtype]->bbHeight()))-bb_station_Station::bbcollisionRadius)/2.000000f));
									if(bb_collision_Collision::bbIntersectRect(bba->bbx-bb_asteroid_Asteroid::bbcollisionRadius/2.000000f,bba->bby-bb_asteroid_Asteroid::bbcollisionRadius/2.000000f,bb_asteroid_Asteroid::bbcollisionRadius,bb_asteroid_Asteroid::bbcollisionRadius,float(bbx),float(bby),bb_station_Station::bbcollisionRadius,bb_station_Station::bbcollisionRadius)){
										bbobj->bbDestroy();
										bbexplode=true;
										bbasteroid->bbExplode();
									}
								}
							}
						}
					}
					if(bba->bbIsDestroyed()){
						break;
					}
				}
				if(bba->bbIsDestroyed()){
					continue;
				}
			}
		}
	}
	if(bbexplode){
		bb_soundmanager_SoundManager::bbPlay(1);
	}
	if(bbstationsLeft<=0){
		bbClearLevel();
		bbstate=3;
		bbtimer=0;
	}
}
void bb_scenegame_SceneGame::bbOnUpdate(){
	bbtimer+=1;
	int bb=bbstate;
	if(bb==0){
		if(bbtimer>bblaunchDelay){
			if(bblaunchCount>0){
				for(int bbi=1;bbi<=bblaunchAmount;bbi=bbi+1){
					bb_asteroid_Asteroid::bbCreate(int(bb_random_Rnd2(50.000000f,float(bb_graphics_DeviceWidth()-100))),int(bb_random_Rnd2(-92.000000f,-32.000000f)),bb_random_Rnd2(-bblaunchDx,bblaunchDx),bb_random_Rnd2(0.1f,bblaunchDy));
				}
				bblaunchCount-=1;
			}else{
				if(bb_asteroid_Asteroid::bbcount<=0){
					bbstate=2;
					bbClearLevel();
				}
			}
			bbtimer=0;
		}
		bbCheckCollision();
		bb_gameobject_GameObject::bbUpdateAll();
	}else{
		if(bb==2){
			if(bbtimer>30 && ((bb_input_MouseHit(0))!=0)){
				bbstate=0;
				bblevel+=1;
				bbInitLevel(bblevel);
			}
		}else{
			if(bb==1){
				if((bb_input_MouseHit(0))!=0){
					bbstate=0;
				}
			}else{
				if(bb==3){
					if(bbtimer>30 && ((bb_input_MouseHit(0))!=0)){
						bb_application_Application::bbGetInstance()->bbSetNextScene(T("title"));
					}
				}
			}
		}
	}
}
void bb_scenegame_SceneGame::bbRenderStateMessage(String bbt,bool bbblink){
	bb_graphics_SetColor(0.000000f,0.000000f,0.000000f);
	bb_graphics_SetAlpha(0.6f);
	bb_graphics_DrawRect(0.000000f,0.000000f,float(bb_graphics_DeviceWidth()),float(bb_graphics_DeviceHeight()));
	bb_graphics_SetAlpha(1.000000f);
	bb_graphics_SetColor(255.000000f,255.000000f,255.000000f);
	bb_angelfont_AngelFont* bbf=bb_gamefonts_GameFonts::bbnormal;
	if(!bbblink || bb_app_Millisecs()/1000 % 2==0){
		bbf->bbDrawText(bbt,bb_graphics_DeviceWidth()/2-bbf->bbTextWidth(bbt)/2,175);
	}
}
String bb_scenegame_SceneGame::bbGetScore(){
	String bbscoreTxt=T("");
	for(int bbi=0;bbi<=5-String(bbscore).Length();bbi=bbi+1){
		bbscoreTxt=T("0")+bbscoreTxt;
	}
	bbscoreTxt+=String(bbscore);
	return bbscoreTxt;
}
void bb_scenegame_SceneGame::bbRenderScore(){
	bb_gamefonts_GameFonts::bbnormal->bbDrawText(bbGetScore(),10,10);
	bb_gamefonts_GameFonts::bbnormal->bbDrawText(String(bblevel+1),bb_graphics_DeviceWidth()-bb_gamefonts_GameFonts::bbnormal->bbTextWidth(String(bblevel))-15,10);
	for(int bbi=1;bbi<=bb_bullet_Bullet::bbbullets-bb_bullet_Bullet::bbbulletsInUse;bbi=bbi+1){
		bb_graphics_DrawImage(bb_bullet_Bullet::bbimg,float(bb_graphics_DeviceWidth()-16*bbi),float(bb_graphics_DeviceHeight()-16),0);
	}
}
void bb_scenegame_SceneGame::bbOnRender(){
	bb_graphics_DrawImage(bbbackground[bblevel % 2],0.000000f,0.000000f,0);
	bb_gameobject_GameObject::bbRenderAll();
	int bb=bbstate;
	if(bb==2){
		bbRenderStateMessage(T("GET READY"),true);
		String bbt=T("Stage cleared!");
		bb_gamefonts_GameFonts::bbnormal->bbDrawText(bbt,bb_graphics_DeviceWidth()/2-bb_gamefonts_GameFonts::bbnormal->bbTextWidth(bbt)/2,135);
	}else{
		if(bb==1){
			bbRenderStateMessage(T("GET READY"),true);
		}else{
			if(bb==3){
				bbRenderStateMessage(T("Your Score: ")+bbGetScore(),true);
				String bbt2=T("G A M E     O V E R");
				bb_gamefonts_GameFonts::bbnormal->bbDrawText(bbt2,bb_graphics_DeviceWidth()/2-bb_gamefonts_GameFonts::bbnormal->bbTextWidth(bbt2)/2,135);
			}
		}
	}
	bbRenderScore();
}
void bb_scenegame_SceneGame::bbOnLeave(){
	bb_gameobject_GameObject::bblist->bbClear();
	bbbackground[0]=0;
	bbbackground[1]=0;
}
void bb_scenegame_SceneGame::mark(){
	bb_scene_Scene::mark();
	gc_mark(bbbackground);
}
bb_list_Enumerator::bb_list_Enumerator(){
	bb_list=0;
	bb_curr=0;
}
bb_list_Enumerator* bb_list_Enumerator::bbnew(bb_list_List* bblist){
	bb_list=bblist;
	bb_curr=bblist->bb_head->bb_succ;
	return this;
}
bb_list_Enumerator* bb_list_Enumerator::bbnew2(){
	return this;
}
bool bb_list_Enumerator::bbHasNext(){
	return bb_curr!=bb_list->bb_head;
}
Object* bb_list_Enumerator::bbNextObject(){
	Object* bbdata=bb_curr->bb_data;
	bb_curr=bb_curr->bb_succ;
	return bbdata;
}
void bb_list_Enumerator::mark(){
	Object::mark();
	gc_mark(bb_list);
	gc_mark(bb_curr);
}
int bb_Main(){
	bb_application_Application* bbapp=bb_application_Application::bbGetInstance();
	bbapp->bbInit();
	bbapp->bbAddFader(((new bb_faderbrightness_FaderBrightness())->bbnew()));
	bb_soundmanager_SoundManager::bbLoadSounds();
	bb_gamefonts_GameFonts::bbLoad();
	bbapp->bbAddScene(T("logo"),((new bb_scenelogo_SceneLogo())->bbnew()));
	bbapp->bbAddScene(T("title"),((new bb_scenetitle_SceneTitle())->bbnew()));
	bbapp->bbAddScene(T("game"),((new bb_scenegame_SceneGame())->bbnew()));
	bbapp->bbRun();
	return 0;
}
int bb_graphics_SetMatrix(float bbix,float bbiy,float bbjx,float bbjy,float bbtx,float bbty){
	bb_graphics_context->bbix=bbix;
	bb_graphics_context->bbiy=bbiy;
	bb_graphics_context->bbjx=bbjx;
	bb_graphics_context->bbjy=bbjy;
	bb_graphics_context->bbtx=bbtx;
	bb_graphics_context->bbty=bbty;
	bb_graphics_context->bbtformed=((bbix!=1.000000f || bbiy!=0.000000f || bbjx!=0.000000f || bbjy!=1.000000f || bbtx!=0.000000f || bbty!=0.000000f)?1:0);
	bb_graphics_context->bbmatDirty=1;
	return 0;
}
int bb_graphics_SetMatrix2(Array<float > bbm){
	bb_graphics_SetMatrix(bbm[0],bbm[1],bbm[2],bbm[3],bbm[4],bbm[5]);
	return 0;
}
int bb_graphics_SetColor(float bbr,float bbg,float bbb){
	bb_graphics_context->bbcolor_r=bbr;
	bb_graphics_context->bbcolor_g=bbg;
	bb_graphics_context->bbcolor_b=bbb;
	bb_graphics_context->bbdevice->SetColor(bbr,bbg,bbb);
	return 0;
}
int bb_graphics_SetAlpha(float bbalpha){
	bb_graphics_context->bbalpha=bbalpha;
	bb_graphics_context->bbdevice->SetAlpha(bbalpha);
	return 0;
}
int bb_graphics_SetBlend(int bbblend){
	bb_graphics_context->bbblend=bbblend;
	bb_graphics_context->bbdevice->SetBlend(bbblend);
	return 0;
}
int bb_graphics_DeviceWidth(){
	return bb_graphics_context->bbdevice->Width();
}
int bb_graphics_DeviceHeight(){
	return bb_graphics_context->bbdevice->Height();
}
int bb_graphics_SetScissor(float bbx,float bby,float bbwidth,float bbheight){
	bb_graphics_context->bbscissor_x=bbx;
	bb_graphics_context->bbscissor_y=bby;
	bb_graphics_context->bbscissor_width=bbwidth;
	bb_graphics_context->bbscissor_height=bbheight;
	bb_graphics_context->bbdevice->SetScissor(int(bbx),int(bby),int(bbwidth),int(bbheight));
	return 0;
}
int bb_graphics_BeginRender(){
	bb_graphics_context->bbmatrixSp=0;
	bb_graphics_SetMatrix(1.000000f,0.000000f,0.000000f,1.000000f,0.000000f,0.000000f);
	bb_graphics_SetColor(255.000000f,255.000000f,255.000000f);
	bb_graphics_SetAlpha(1.000000f);
	bb_graphics_SetBlend(0);
	bb_graphics_SetScissor(0.000000f,0.000000f,float(bb_graphics_DeviceWidth()),float(bb_graphics_DeviceHeight()));
	return 0;
}
int bb_graphics_EndRender(){
	return 0;
}
bb_gameobject_GameObject::bb_gameobject_GameObject(){
	bbx=0;
	bby=0;
}
void bb_gameobject_GameObject::bbInit(){
}
bb_list_List* bb_gameobject_GameObject::bblist;
bb_gameobject_GameObject* bb_gameobject_GameObject::bbnew(){
	if(bblist==0){
		bblist=(new bb_list_List())->bbnew();
	}
	bblist->bbAddLast(this);
	return this;
}
bool bb_gameobject_GameObject::bbIsDestroyed(){
	return false;
}
void bb_gameobject_GameObject::bbDestroy(){
}
void bb_gameobject_GameObject::bbOnUpdate(){
}
void bb_gameobject_GameObject::bbUpdateAll(){
	bb_list_Enumerator* bb=bblist->bbObjectEnumerator();
	while(bb->bbHasNext()){
		bb_gameobject_GameObject* bbobj=dynamic_cast<bb_gameobject_GameObject*>((Object*)(bb->bbNextObject()));
		if(!bbobj->bbIsDestroyed()){
			bbobj->bbOnUpdate();
		}
	}
}
void bb_gameobject_GameObject::bbOnRender(){
}
void bb_gameobject_GameObject::bbRenderAll(){
	bb_list_Enumerator* bb=bblist->bbObjectEnumerator();
	while(bb->bbHasNext()){
		bb_gameobject_GameObject* bbobj=dynamic_cast<bb_gameobject_GameObject*>((Object*)(bb->bbNextObject()));
		if(!bbobj->bbIsDestroyed()){
			bbobj->bbOnRender();
		}
	}
}
void bb_gameobject_GameObject::mark(){
	Object::mark();
}
bb_base_Base::bb_base_Base(){
}
bb_graphics_Image* bb_base_Base::bbimg;
void bb_base_Base::bbInit(){
	if(bbimg==0){
		bbimg=bb_graphics_LoadImage(T("gfx/base.png"),1,bb_graphics_Image::bbDefaultFlags);
	}
}
bb_base_Base* bb_base_Base::bbnew(){
	bb_gameobject_GameObject::bbnew();
	bbx=135.000000f;
	bby=249.000000f;
	return this;
}
bb_base_Base* bb_base_Base::bbCreate(){
	return (new bb_base_Base())->bbnew();
}
void bb_base_Base::bbOnRender(){
	bb_graphics_DrawImage(bbimg,bbx,bby,0);
}
void bb_base_Base::bbOnUpdate(){
	if((bb_input_MouseHit(0))!=0){
		int bbx=int(bb_input_MouseX());
		int bby=int(bb_input_MouseY());
		bb_bullet_Bullet::bbCreate(bbx,bby);
	}
}
void bb_base_Base::bbDestroy(){
	bb_gameobject_GameObject::bblist->bbRemove(this);
}
void bb_base_Base::mark(){
	bb_gameobject_GameObject::mark();
}
bb_station_Station::bb_station_Station(){
	bbdestroyed=false;
	bbtype=0;
}
Array<bb_graphics_Image* > bb_station_Station::bbimg;
float bb_station_Station::bbcollisionRadius;
void bb_station_Station::bbInit(){
	if(bbimg[0]==0){
		bbimg[0]=bb_graphics_LoadImage(T("gfx/station.png"),1,bb_graphics_Image::bbDefaultFlags);
		bbimg[1]=bb_graphics_LoadImage(T("gfx/station2.png"),1,bb_graphics_Image::bbDefaultFlags);
		bbcollisionRadius=float(bbimg[0]->bbWidth())*0.7f;
	}
}
bb_station_Station* bb_station_Station::bbnew(){
	bb_gameobject_GameObject::bbnew();
	return this;
}
void bb_station_Station::bbCreate(int bbx,int bby,int bbtype){
	bb_station_Station* bbs=(new bb_station_Station())->bbnew();
	bbs->bbdestroyed=false;
	bbs->bbx=float(bbx);
	bbs->bby=float(bby);
	bbs->bbtype=bbtype;
}
bool bb_station_Station::bbIsDestroyed(){
	return bbdestroyed;
}
void bb_station_Station::bbOnRender(){
	bb_graphics_DrawImage(bbimg[bbtype],bbx,bby,0);
}
void bb_station_Station::bbOnUpdate(){
}
void bb_station_Station::bbDestroy(){
	bb_explosion_Explosion::bbCreate(int(bbx+float(bbimg[bbtype]->bbWidth()/2)),int(bby+15.000000f),1);
	bbdestroyed=true;
}
void bb_station_Station::mark(){
	bb_gameobject_GameObject::mark();
}
bb_bullet_Bullet::bb_bullet_Bullet(){
	bbstate=0;
	bbsteps=0;
	bbdx=0;
	bbdy=0;
}
bb_graphics_Image* bb_bullet_Bullet::bbimg;
int bb_bullet_Bullet::bbcollisionRadius;
int bb_bullet_Bullet::bbbullets;
int bb_bullet_Bullet::bbbulletsInUse;
void bb_bullet_Bullet::bbInit(){
	if(bbimg==0){
		bbimg=bb_graphics_LoadImage(T("gfx/bullet.png"),1,1);
		bbcollisionRadius=bbimg->bbWidth()/2;
	}
	bbbullets=1;
	bbbulletsInUse=0;
}
void bb_bullet_Bullet::bbDestroy(){
	bbbulletsInUse-=1;
	if(bbbulletsInUse<0){
		bbbulletsInUse=0;
	}
	bbstate=1;
}
bb_bullet_Bullet* bb_bullet_Bullet::bbnew(){
	bb_gameobject_GameObject::bbnew();
	return this;
}
int bb_bullet_Bullet::bbLAUNCH_X;
int bb_bullet_Bullet::bbLAUNCH_Y;
int bb_bullet_Bullet::bbSPEED;
bb_bullet_Bullet* bb_bullet_Bullet::bbCreate(int bbdx,int bbdy){
	if(bbbulletsInUse>=bbbullets){
		return 0;
	}
	bbbulletsInUse+=1;
	bb_bullet_Bullet* bbb=0;
	bb_list_Enumerator* bb=bb_gameobject_GameObject::bblist->bbObjectEnumerator();
	while(bb->bbHasNext()){
		bb_gameobject_GameObject* bbgo=dynamic_cast<bb_gameobject_GameObject*>((Object*)(bb->bbNextObject()));
		if(((dynamic_cast<bb_bullet_Bullet*>((bb_gameobject_GameObject*)(bbgo)))!=0)){
			bbb=dynamic_cast<bb_bullet_Bullet*>((bb_gameobject_GameObject*)(bbgo));
			if(bbb->bbstate==1){
				break;
			}
			bbb=0;
		}
	}
	if(!((bbb)!=0)){
		bbb=(new bb_bullet_Bullet())->bbnew();
	}
	bbb->bbx=float(bbLAUNCH_X);
	bbb->bby=float(bbLAUNCH_Y);
	bbb->bbsteps=int(((float)sqrt((float)pow(float(bbdx)-bbb->bbx,2.000000f)+(float)pow(float(bbdy)-bbb->bby,2.000000f))/float(bbSPEED)));
	bbb->bbdx=(float(bbdx)-bbb->bbx)/float(bbb->bbsteps);
	bbb->bbdy=(float(bbdy)-bbb->bby)/float(bbb->bbsteps);
	bbb->bbstate=0;
	bb_soundmanager_SoundManager::bbPlay(0);
	return bbb;
}
bool bb_bullet_Bullet::bbIsDestroyed(){
	return bbstate==1;
}
void bb_bullet_Bullet::bbOnUpdate(){
	if(bbstate==1){
		return;
	}
	if(bbstate==0){
		bbx+=bbdx;
		bby+=bbdy;
		bbsteps-=1;
		if(bbsteps<0){
			bbDestroy();
			bb_explosion_Explosion::bbCreate(int(bbx),int(bby),0);
			bb_soundmanager_SoundManager::bbPlay(1);
		}
	}
	if(bbx<-32.000000f || bby<-32.000000f || bbx>float(bb_graphics_DeviceWidth()+32) || bby>float(bb_graphics_DeviceHeight()+32)){
		bbDestroy();
	}
}
void bb_bullet_Bullet::bbOnRender(){
	if(bbstate==1){
		return;
	}
	bb_graphics_SetColor(0.000000f,255.000000f,255.000000f);
	bb_graphics_DrawLine(float(bbLAUNCH_X),float(bbLAUNCH_Y),bbx,bby);
	bb_graphics_SetColor(255.000000f,255.000000f,255.000000f);
	bb_graphics_DrawImage(bbimg,bbx,bby,0);
}
void bb_bullet_Bullet::bbPowerUp(){
	bbbullets+=1;
	if(bbbullets>=8){
		bbbullets=8;
	}
}
void bb_bullet_Bullet::mark(){
	bb_gameobject_GameObject::mark();
}
bb_explosion_Explosion::bb_explosion_Explosion(){
	bbframe=0;
	bbtyp=0;
}
Array<bb_graphics_Image* > bb_explosion_Explosion::bbimg;
float bb_explosion_Explosion::bbcollisionRadius;
void bb_explosion_Explosion::bbInit(){
	if(bbimg[0]==0 || bbimg[1]==0){
		bbimg[0]=bb_graphics_LoadImage(T("gfx/explosion3.png"),1,bb_graphics_Image::bbDefaultFlags);
		bbimg[0]=bbimg[0]->bbGrabImage(0,0,128,128,64,1);
		bbimg[1]=bb_graphics_LoadImage(T("gfx/mushroom.png"),1,bb_graphics_Image::bbDefaultFlags);
		bbimg[1]=bbimg[1]->bbGrabImage(0,0,128,128,64,1);
		bbcollisionRadius=float(bbimg[0]->bbWidth())*0.25f;
	}
}
void bb_explosion_Explosion::bbDestroy(){
	bbframe=63.000000f;
}
bool bb_explosion_Explosion::bbIsDestroyed(){
	return bbframe>=63.000000f;
}
bb_explosion_Explosion* bb_explosion_Explosion::bbnew(){
	bb_gameobject_GameObject::bbnew();
	return this;
}
bb_explosion_Explosion* bb_explosion_Explosion::bbCreate(int bbx,int bby,int bbtyp){
	bb_explosion_Explosion* bbe=0;
	bb_list_Enumerator* bb=bb_gameobject_GameObject::bblist->bbObjectEnumerator();
	while(bb->bbHasNext()){
		bb_gameobject_GameObject* bbgo=dynamic_cast<bb_gameobject_GameObject*>((Object*)(bb->bbNextObject()));
		bb_explosion_Explosion* bbel=dynamic_cast<bb_explosion_Explosion*>((bb_gameobject_GameObject*)(bbgo));
		if(((bbel)!=0) && bbel->bbIsDestroyed()){
			bbel->bbx=float(bbx);
			bbel->bby=float(bby);
			bbel->bbframe=0.000000f;
			bbel->bbtyp=bbtyp;
			return bbel;
		}
	}
	bbe=(new bb_explosion_Explosion())->bbnew();
	bbe->bbx=float(bbx);
	bbe->bby=float(bby);
	bbe->bbtyp=bbtyp;
	bbe->bbframe=0.000000f;
	return bbe;
}
void bb_explosion_Explosion::bbOnUpdate(){
	if(bbIsDestroyed()){
		return;
	}
	bbframe+=1.000000f;
}
void bb_explosion_Explosion::bbOnRender(){
	if(bbIsDestroyed()){
		return;
	}
	bb_graphics_DrawImage(bbimg[bbtyp],bbx,bby,int(bbframe));
}
void bb_explosion_Explosion::mark(){
	bb_gameobject_GameObject::mark();
}
bb_asteroid_Asteroid::bb_asteroid_Asteroid(){
	bbstate=0;
	bbsx=0;
	bbsy=0;
	bbdx=0;
	bbdy=0;
	bbframe=0;
}
bb_graphics_Image* bb_asteroid_Asteroid::bbimg;
float bb_asteroid_Asteroid::bbcollisionRadius;
void bb_asteroid_Asteroid::bbInit(){
	if(bbimg==0){
		bbimg=bb_graphics_LoadImage(T("gfx/asteroids.png"),1,bb_graphics_Image::bbDefaultFlags);
		bbimg=bbimg->bbGrabImage(0,0,32,32,32,1);
		bbcollisionRadius=float(bbimg->bbWidth()/2)*0.7f;
	}
}
bool bb_asteroid_Asteroid::bbIsDestroyed(){
	return bbstate==1;
}
bb_asteroid_Asteroid* bb_asteroid_Asteroid::bbnew(){
	bb_gameobject_GameObject::bbnew();
	return this;
}
int bb_asteroid_Asteroid::bbcount;
bb_asteroid_Asteroid* bb_asteroid_Asteroid::bbCreate(int bbx,int bby,float bbdx,float bbdy){
	bb_asteroid_Asteroid* bba=0;
	bb_list_Enumerator* bb=bb_gameobject_GameObject::bblist->bbObjectEnumerator();
	while(bb->bbHasNext()){
		bb_gameobject_GameObject* bbgo=dynamic_cast<bb_gameobject_GameObject*>((Object*)(bb->bbNextObject()));
		if(((dynamic_cast<bb_asteroid_Asteroid*>((bb_gameobject_GameObject*)(bbgo)))!=0)){
			bba=dynamic_cast<bb_asteroid_Asteroid*>((bb_gameobject_GameObject*)(bbgo));
			if(bba->bbIsDestroyed()){
				break;
			}
			bba=0;
		}
	}
	if(!((bba)!=0)){
		bba=(new bb_asteroid_Asteroid())->bbnew();
	}
	bba->bbx=float(bbx);
	bba->bby=float(bby);
	bba->bbsx=float(bbx);
	bba->bbsy=float(bby);
	bba->bbdx=bbdx;
	bba->bbdy=bbdy;
	bba->bbstate=0;
	bbcount+=1;
	return bba;
}
void bb_asteroid_Asteroid::bbDestroy(){
	bbstate=1;
	bbcount-=1;
	if(bbcount<=0){
		bbcount=0;
	}
}
void bb_asteroid_Asteroid::bbExplode(){
	bbDestroy();
	bb_explosion_Explosion::bbCreate(int(bbx),int(bby),0);
}
void bb_asteroid_Asteroid::bbOnUpdate(){
	bbframe+=0.5f;
	if(bbframe>=32.000000f){
		bbframe-=32.000000f;
	}
	bbx+=bbdx;
	bby+=bbdy;
	if(bbx<-32.000000f || bbx>float(bb_graphics_DeviceWidth()+32) || bby>float(bb_graphics_DeviceHeight()+32)){
		bbDestroy();
	}
}
void bb_asteroid_Asteroid::bbOnRender(){
	bb_graphics_SetColor(192.000000f,0.000000f,0.000000f);
	bb_graphics_SetAlpha(0.5f);
	bb_graphics_DrawLine(bbsx,bbsy,bbx,bby);
	bb_graphics_SetAlpha(1.000000f);
	bb_graphics_SetColor(255.000000f,255.000000f,255.000000f);
	bb_graphics_DrawImage(bbimg,bbx,bby,int(bbframe));
}
void bb_asteroid_Asteroid::mark(){
	bb_gameobject_GameObject::mark();
}
bb_powerup_PowerUp::bb_powerup_PowerUp(){
	bbdestroyed=false;
	bbdx=0;
	bbdy=0;
	bbframe=0;
}
bb_graphics_Image* bb_powerup_PowerUp::bbimg;
void bb_powerup_PowerUp::bbInit(){
	if(bbimg==0){
		bbimg=bb_graphics_LoadImage(T("gfx/powerup1.png"),1,bb_graphics_Image::bbDefaultFlags);
		bbimg=bbimg->bbGrabImage(0,0,16,16,32,1);
	}
}
bool bb_powerup_PowerUp::bbIsDestroyed(){
	return bbdestroyed;
}
bb_powerup_PowerUp* bb_powerup_PowerUp::bbnew(){
	bb_gameobject_GameObject::bbnew();
	return this;
}
bb_powerup_PowerUp* bb_powerup_PowerUp::bbCreate(float bbx,float bby,float bbdx,float bbdy){
	bb_powerup_PowerUp* bbp=0;
	bb_list_Enumerator* bb=bb_gameobject_GameObject::bblist->bbObjectEnumerator();
	while(bb->bbHasNext()){
		bb_gameobject_GameObject* bbgo=dynamic_cast<bb_gameobject_GameObject*>((Object*)(bb->bbNextObject()));
		if(((dynamic_cast<bb_powerup_PowerUp*>((bb_gameobject_GameObject*)(bbgo)))!=0)){
			bbp=dynamic_cast<bb_powerup_PowerUp*>((bb_gameobject_GameObject*)(bbgo));
			if(bbp->bbIsDestroyed()){
				break;
			}
			bbp=0;
		}
	}
	if(!((bbp)!=0)){
		bbp=(new bb_powerup_PowerUp())->bbnew();
	}
	bbp->bbx=bbx;
	bbp->bby=bby;
	bbp->bbdx=bbdx;
	bbp->bbdy=bbdy;
	bbp->bbdestroyed=false;
	return bbp;
}
void bb_powerup_PowerUp::bbOnRender(){
	bb_graphics_DrawImage(bbimg,bbx,bby,int(bbframe));
}
void bb_powerup_PowerUp::bbDestroy(){
	bbdestroyed=true;
}
void bb_powerup_PowerUp::bbOnUpdate(){
	if(bbIsDestroyed()){
		return;
	}
	if((bb_input_MouseDown(0))!=0){
		int bbmx=int(bb_input_MouseX());
		int bbmy=int(bb_input_MouseY());
		if(bb_collision_Collision::bbIntersectCircle(float(bbmx),float(bbmy),16.000000f,bbx,bby,16.000000f)){
			bb_bullet_Bullet::bbPowerUp();
			bb_scenegame_SceneGame* bbs=dynamic_cast<bb_scenegame_SceneGame*>((bb_scene_Scene*)(bb_application_Application::bbGetInstance()->bbcurrentScene));
			bbs->bbscore+=125;
			bbDestroy();
			return;
		}
	}
	bbframe+=0.000000f;
	if(bbframe>=32.000000f){
		bbframe-=32.000000f;
	}
	bbx+=bbdx;
	bby+=bbdy;
	if(bbdy<4.000000f){
		bbdy+=0.05f;
	}
	if(bby>float(bb_graphics_DeviceHeight()+32)){
		bbDestroy();
	}
}
void bb_powerup_PowerUp::mark(){
	bb_gameobject_GameObject::mark();
}
int bb_random_Seed;
int bb_app_SetUpdateRate(int bbhertz){
	return bb_app_device->SetUpdateRate(bbhertz);
}
int bb_graphics_SetFont(bb_graphics_Image* bbfont,int bbfirstChar){
	if(!((bbfont)!=0)){
		if(!((bb_graphics_context->bbdefaultFont)!=0)){
			bb_graphics_context->bbdefaultFont=bb_graphics_LoadImage2(T("mojo_font.png"),7,13,256,0);
		}
		bbfont=bb_graphics_context->bbdefaultFont;
		bbfirstChar=32;
	}
	bb_graphics_context->bbfont=bbfont;
	bb_graphics_context->bbfirstChar=bbfirstChar;
	return 0;
}
int bb_graphics_PushMatrix(){
	int bbsp=bb_graphics_context->bbmatrixSp;
	bb_graphics_context->bbmatrixStack[bbsp+0]=bb_graphics_context->bbix;
	bb_graphics_context->bbmatrixStack[bbsp+1]=bb_graphics_context->bbiy;
	bb_graphics_context->bbmatrixStack[bbsp+2]=bb_graphics_context->bbjx;
	bb_graphics_context->bbmatrixStack[bbsp+3]=bb_graphics_context->bbjy;
	bb_graphics_context->bbmatrixStack[bbsp+4]=bb_graphics_context->bbtx;
	bb_graphics_context->bbmatrixStack[bbsp+5]=bb_graphics_context->bbty;
	bb_graphics_context->bbmatrixSp=bbsp+6;
	return 0;
}
int bb_graphics_Transform(float bbix,float bbiy,float bbjx,float bbjy,float bbtx,float bbty){
	float bbix2=bbix*bb_graphics_context->bbix+bbiy*bb_graphics_context->bbjx;
	float bbiy2=bbix*bb_graphics_context->bbiy+bbiy*bb_graphics_context->bbjy;
	float bbjx2=bbjx*bb_graphics_context->bbix+bbjy*bb_graphics_context->bbjx;
	float bbjy2=bbjx*bb_graphics_context->bbiy+bbjy*bb_graphics_context->bbjy;
	float bbtx2=bbtx*bb_graphics_context->bbix+bbty*bb_graphics_context->bbjx+bb_graphics_context->bbtx;
	float bbty2=bbtx*bb_graphics_context->bbiy+bbty*bb_graphics_context->bbjy+bb_graphics_context->bbty;
	bb_graphics_SetMatrix(bbix2,bbiy2,bbjx2,bbjy2,bbtx2,bbty2);
	return 0;
}
Array<float > bb_graphics_Transform2(Array<float > bbcoords){
	Array<float > bbout=Array<float >(bbcoords.Length());
	for(int bbi=0;bbi<bbcoords.Length()-1;bbi=bbi+2){
		float bbx=bbcoords[bbi];
		float bby=bbcoords[bbi+1];
		bbout[bbi]=bbx*bb_graphics_context->bbix+bby*bb_graphics_context->bbjx+bb_graphics_context->bbtx;
		bbout[bbi+1]=bbx*bb_graphics_context->bbiy+bby*bb_graphics_context->bbjy+bb_graphics_context->bbty;
	}
	return bbout;
}
int bb_graphics_Translate(float bbx,float bby){
	bb_graphics_Transform(1.000000f,0.000000f,0.000000f,1.000000f,bbx,bby);
	return 0;
}
int bb_graphics_ValidateMatrix(){
	if((bb_graphics_context->bbmatDirty)!=0){
		bb_graphics_context->bbdevice->SetMatrix(bb_graphics_context->bbix,bb_graphics_context->bbiy,bb_graphics_context->bbjx,bb_graphics_context->bbjy,bb_graphics_context->bbtx,bb_graphics_context->bbty);
		bb_graphics_context->bbmatDirty=0;
	}
	return 0;
}
int bb_graphics_PopMatrix(){
	int bbsp=bb_graphics_context->bbmatrixSp-6;
	bb_graphics_SetMatrix(bb_graphics_context->bbmatrixStack[bbsp+0],bb_graphics_context->bbmatrixStack[bbsp+1],bb_graphics_context->bbmatrixStack[bbsp+2],bb_graphics_context->bbmatrixStack[bbsp+3],bb_graphics_context->bbmatrixStack[bbsp+4],bb_graphics_context->bbmatrixStack[bbsp+5]);
	bb_graphics_context->bbmatrixSp=bbsp;
	return 0;
}
int bb_graphics_DrawImage(bb_graphics_Image* bbimage,float bbx,float bby,int bbframe){
	bb_graphics_Frame* bbf=bbimage->bbframes[bbframe];
	if((bb_graphics_context->bbtformed)!=0){
		bb_graphics_PushMatrix();
		bb_graphics_Translate(bbx-bbimage->bbtx,bby-bbimage->bbty);
		bb_graphics_ValidateMatrix();
		if((bbimage->bbflags&65536)!=0){
			bb_graphics_context->bbdevice->DrawSurface(bbimage->bbsurface,0.000000f,0.000000f);
		}else{
			bb_graphics_context->bbdevice->DrawSurface2(bbimage->bbsurface,0.000000f,0.000000f,bbf->bbx,bbf->bby,bbimage->bbwidth,bbimage->bbheight);
		}
		bb_graphics_PopMatrix();
	}else{
		bb_graphics_ValidateMatrix();
		if((bbimage->bbflags&65536)!=0){
			bb_graphics_context->bbdevice->DrawSurface(bbimage->bbsurface,bbx-bbimage->bbtx,bby-bbimage->bbty);
		}else{
			bb_graphics_context->bbdevice->DrawSurface2(bbimage->bbsurface,bbx-bbimage->bbtx,bby-bbimage->bbty,bbf->bbx,bbf->bby,bbimage->bbwidth,bbimage->bbheight);
		}
	}
	return 0;
}
int bb_graphics_Rotate(float bbangle){
	bb_graphics_Transform((float)cos((bbangle)*0.0174532925),-(float)sin((bbangle)*0.0174532925),(float)sin((bbangle)*0.0174532925),(float)cos((bbangle)*0.0174532925),0.000000f,0.000000f);
	return 0;
}
int bb_graphics_Scale(float bbx,float bby){
	bb_graphics_Transform(bbx,0.000000f,0.000000f,bby,0.000000f,0.000000f);
	return 0;
}
int bb_graphics_DrawImage2(bb_graphics_Image* bbimage,float bbx,float bby,float bbrotation,float bbscaleX,float bbscaleY,int bbframe){
	bb_graphics_Frame* bbf=bbimage->bbframes[bbframe];
	bb_graphics_PushMatrix();
	bb_graphics_Translate(bbx,bby);
	bb_graphics_Rotate(bbrotation);
	bb_graphics_Scale(bbscaleX,bbscaleY);
	bb_graphics_Translate(-bbimage->bbtx,-bbimage->bbty);
	bb_graphics_ValidateMatrix();
	if((bbimage->bbflags&65536)!=0){
		bb_graphics_context->bbdevice->DrawSurface(bbimage->bbsurface,0.000000f,0.000000f);
	}else{
		bb_graphics_context->bbdevice->DrawSurface2(bbimage->bbsurface,0.000000f,0.000000f,bbf->bbx,bbf->bby,bbimage->bbwidth,bbimage->bbheight);
	}
	bb_graphics_PopMatrix();
	return 0;
}
int bb_graphics_DrawText(String bbtext,float bbx,float bby,float bbxalign,float bbyalign){
	if(!((bb_graphics_context->bbfont)!=0)){
		bb_graphics_SetFont(0,32);
	}
	if(!((bb_graphics_context->bbfont)!=0)){
		return 0;
	}
	if(!((bb_graphics_context->bbfont->bbLoaded())!=0)){
		return 0;
	}
	bbx=bbx-float((bb_graphics_context->bbfont->bbWidth()*bbtext.Length()))*bbxalign;
	bby=bby-float(bb_graphics_context->bbfont->bbHeight())*bbyalign;
	for(int bbi=0;bbi<bbtext.Length();bbi=bbi+1){
		int bbch=(int)bbtext[bbi]-bb_graphics_context->bbfirstChar;
		if(bbch>=0 && bbch<bb_graphics_context->bbfont->bbFrames()){
			bb_graphics_DrawImage(bb_graphics_context->bbfont,bbx+float(bbi*bb_graphics_context->bbfont->bbWidth()),bby,bbch);
		}
	}
	return 0;
}
float bb_graphics_GetAlpha(){
	return bb_graphics_context->bbalpha;
}
Array<float > bb_graphics_GetColor(){
	float bb[]={bb_graphics_context->bbcolor_r,bb_graphics_context->bbcolor_g,bb_graphics_context->bbcolor_b};
	return Array<float >(bb,3);
}
int bb_graphics_DrawRect(float bbx,float bby,float bbw,float bbh){
	bb_graphics_ValidateMatrix();
	bb_graphics_context->bbdevice->DrawRect(bbx,bby,bbw,bbh);
	return 0;
}
int bb_app_Millisecs(){
	return bb_app_device->MilliSecs();
}
int bb_audio_PlaySound(bb_audio_Sound* bbsound,int bbchannel,int bbflags){
	if((bbsound->bbsample)!=0){
		bb_audio_device->PlaySample(bbsound->bbsample,bbchannel,bbflags);
	}
	return 0;
}
int bb_input_MouseHit(int bbbutton){
	return bb_input_device->KeyHit(1+bbbutton);
}
int bb_graphics_DrawImageRect(bb_graphics_Image* bbimage,float bbx,float bby,int bbsrcX,int bbsrcY,int bbsrcWidth,int bbsrcHeight,int bbframe){
	bb_graphics_Frame* bbf=bbimage->bbframes[bbframe];
	if((bb_graphics_context->bbtformed)!=0){
		bb_graphics_PushMatrix();
		bb_graphics_Translate(-bbimage->bbtx+bbx,-bbimage->bbty+bby);
		bb_graphics_ValidateMatrix();
		bb_graphics_context->bbdevice->DrawSurface2(bbimage->bbsurface,0.000000f,0.000000f,bbsrcX+bbf->bbx,bbsrcY+bbf->bby,bbsrcWidth,bbsrcHeight);
		bb_graphics_PopMatrix();
	}else{
		bb_graphics_ValidateMatrix();
		bb_graphics_context->bbdevice->DrawSurface2(bbimage->bbsurface,-bbimage->bbtx+bbx,-bbimage->bbty+bby,bbsrcX+bbf->bbx,bbsrcY+bbf->bby,bbsrcWidth,bbsrcHeight);
	}
	return 0;
}
int bb_graphics_DrawImageRect2(bb_graphics_Image* bbimage,float bbx,float bby,int bbsrcX,int bbsrcY,int bbsrcWidth,int bbsrcHeight,float bbrotation,float bbscaleX,float bbscaleY,int bbframe){
	bb_graphics_Frame* bbf=bbimage->bbframes[bbframe];
	bb_graphics_PushMatrix();
	bb_graphics_Translate(bbx,bby);
	bb_graphics_Rotate(bbrotation);
	bb_graphics_Scale(bbscaleX,bbscaleY);
	bb_graphics_Translate(-bbimage->bbtx,-bbimage->bbty);
	bb_graphics_ValidateMatrix();
	bb_graphics_context->bbdevice->DrawSurface2(bbimage->bbsurface,0.000000f,0.000000f,bbsrcX+bbf->bbx,bbsrcY+bbf->bby,bbsrcWidth,bbsrcHeight);
	bb_graphics_PopMatrix();
	return 0;
}
float bb_random_Rnd(){
	bb_random_Seed=bb_random_Seed*1664525+1013904223|0;
	return float(bb_random_Seed/4&1073741823)/1073741824.0000f;
}
float bb_random_Rnd2(float bblow,float bbhigh){
	return bb_random_Rnd3(bbhigh-bblow)+bblow;
}
float bb_random_Rnd3(float bbrange){
	return bb_random_Rnd()*bbrange;
}
bb_collision_Collision::bb_collision_Collision(){
}
bool bb_collision_Collision::bbIntersectCircle(float bbx1,float bby1,float bbr1,float bbx2,float bby2,float bbr2){
	float bbdx=0;
	float bbdy=0;
	float bbdistance=0;
	bbdx=bbx1-bbx2;
	bbdy=bby1-bby2;
	bbdistance=(float)sqrt((float)pow(bbdx,2.000000f)+(float)pow(bbdy,2.000000f))-bbr1-bbr2;
	if(bbdistance<0.0f){
		return true;
	}
	return false;
}
bool bb_collision_Collision::bbIntersectRect(float bbx1,float bby1,float bbw1,float bbh1,float bbx2,float bby2,float bbw2,float bbh2){
	if(bbx1>bbx2+bbw2 || bbx1+bbw1<bbx2){
		return false;
	}
	if(bby1>bby2+bbh2 || bby1+bbh1<bby2){
		return false;
	}
	return true;
}
void bb_collision_Collision::mark(){
	Object::mark();
}
int bb_math_Abs(int bbx){
	if(bbx>=0){
		return bbx;
	}
	return -bbx;
}
float bb_math_Abs2(float bbx){
	if(bbx>=0.000000f){
		return bbx;
	}
	return -bbx;
}
int bb_audio_StopChannel(int bbchannel){
	bb_audio_device->StopChannel(bbchannel);
	return 0;
}
float bb_input_MouseX(){
	return bb_input_device->MouseX();
}
float bb_input_MouseY(){
	return bb_input_device->MouseY();
}
int bb_graphics_DrawLine(float bbx1,float bby1,float bbx2,float bby2){
	bb_graphics_ValidateMatrix();
	bb_graphics_context->bbdevice->DrawLine(bbx1,bby1,bbx2,bby2);
	return 0;
}
int bb_input_MouseDown(int bbbutton){
	return bb_input_device->KeyDown(1+bbbutton);
}
int bb_Init(){
	bb_application_Application::bbapplication=0;
	bb_graphics_context=0;
	bb_input_device=0;
	bb_audio_device=0;
	bb_app_device=0;
	bb_resource_resources=(new bb_map_StringMap())->bbnew();
	bb_soundmanager_SoundManager::bbbackgroundMusic=0;
	bb_soundmanager_SoundManager::bbsfx=Array<bb_audio_Sound* >(2);
	bb_angelfont_AngelFont::bberror=T("");
	bb_angelfont_AngelFont::bbcurrent=0;
	bb_graphics_Image::bbDefaultFlags=0;
	bb_angelfont_AngelFont::bb_list=(new bb_map_StringMap())->bbnew();
	bb_gamefonts_GameFonts::bbnormal=0;
	bb_base_Base::bbimg=0;
	bb_station_Station::bbimg=Array<bb_graphics_Image* >(2);
	bb_station_Station::bbcollisionRadius=0;
	bb_bullet_Bullet::bbimg=0;
	bb_bullet_Bullet::bbcollisionRadius=0;
	bb_bullet_Bullet::bbbullets=0;
	bb_bullet_Bullet::bbbulletsInUse=0;
	bb_explosion_Explosion::bbimg=Array<bb_graphics_Image* >(2);
	bb_explosion_Explosion::bbcollisionRadius=0;
	bb_asteroid_Asteroid::bbimg=0;
	bb_asteroid_Asteroid::bbcollisionRadius=0;
	bb_powerup_PowerUp::bbimg=0;
	bb_gameobject_GameObject::bblist=0;
	bb_random_Seed=1234;
	bb_asteroid_Asteroid::bbcount=0;
	bb_bullet_Bullet::bbLAUNCH_X=166;
	bb_bullet_Bullet::bbLAUNCH_Y=255;
	bb_bullet_Bullet::bbSPEED=2;
	return 0;
}
void gc_mark(){
	gc_mark(bb_application_Application::bbapplication);
	gc_mark(bb_graphics_context);
	gc_mark(bb_input_device);
	gc_mark(bb_audio_device);
	gc_mark(bb_app_device);
	gc_mark(bb_resource_resources);
	gc_mark(bb_soundmanager_SoundManager::bbbackgroundMusic);
	gc_mark(bb_soundmanager_SoundManager::bbsfx);
	gc_mark(bb_angelfont_AngelFont::bbcurrent);
	gc_mark(bb_angelfont_AngelFont::bb_list);
	gc_mark(bb_gamefonts_GameFonts::bbnormal);
	gc_mark(bb_base_Base::bbimg);
	gc_mark(bb_station_Station::bbimg);
	gc_mark(bb_bullet_Bullet::bbimg);
	gc_mark(bb_explosion_Explosion::bbimg);
	gc_mark(bb_asteroid_Asteroid::bbimg);
	gc_mark(bb_powerup_PowerUp::bbimg);
	gc_mark(bb_gameobject_GameObject::bblist);
}
//${END_CODE}

//***** main.m *****

int main(int argc, char *argv[]) {

    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
    UIApplicationMain( argc,argv,nil,nil );
    
    [pool release];
	
	return 0;
}
